AC_DEFUN([AC_CXX_NAMESPACES],
[AC_CACHE_CHECK(whether the compiler implements namespaces,
ac_cv_cxx_namespaces,
[AC_LANG_SAVE
 AC_LANG_CPLUSPLUS
 AC_TRY_COMPILE([namespace Outer { namespace Inner { int i = 0; }}],
                [using namespace Outer::Inner; return i;],
 ac_cv_cxx_namespaces=yes, ac_cv_cxx_namespaces=no)
 AC_LANG_RESTORE
])
if test "$ac_cv_cxx_namespaces" = yes; then
  AC_DEFINE(HAVE_NAMESPACES,,[define if the compiler implements namespaces])
fi
])

AC_DEFUN([AC_CXX_HAVE_STD],
[AC_CACHE_CHECK(whether the compiler supports ISO C++ standard library,
ac_cv_cxx_have_std,
[AC_REQUIRE([AC_CXX_NAMESPACES])
 AC_LANG_SAVE
 AC_LANG_CPLUSPLUS
 AC_TRY_COMPILE([#include <iostream>
#include <map>
#include <iomanip>
#ifdef HAVE_NAMESPACES
using namespace std;
#endif],[return 0;],
 ac_cv_cxx_have_std=yes, ac_cv_cxx_have_std=no)
 AC_LANG_RESTORE
])
if test "$ac_cv_cxx_have_std" = yes; then
  AC_DEFINE(HAVE_STD,,[define if the compiler supports ISO C++ standard library])
fi
])

AC_DEFUN([AC_CXX_HAVE_STD_IOSTREAM],
[
 AC_REQUIRE([AC_CXX_NAMESPACES])
 AC_REQUIRE([AC_CXX_HAVE_STD])

 AC_CACHE_CHECK(whether the compiler supports the new standard iostream library,
  ac_cv_cxx_have_std_iostream,
  [
    AC_LANG_SAVE
    AC_LANG_CPLUSPLUS

    AC_TRY_COMPILE([
      #include <iostream>
      #include <sstream>
      #ifdef HAVE_NAMESPACES
      using namespace std;

      #endif],[
      ios_base::fmtflags flags_;
      stringstream out;
      out.unsetf(ios_base::right|ios_base::internal);
      return 0;],
      ac_cv_cxx_have_std_iostream=yes, ac_cv_cxx_have_std_iostream=no)

		if test "$ac_cv_cxx_have_std_iostream" = no; then

      AC_TRY_COMPILE([
	  		#define __USE_STD_IOSTREAM
        #include <iostream>
        #include <sstream>
        #ifdef HAVE_NAMESPACES
        using namespace std;

        #endif],[
        ios_base::fmtflags flags_;
        stringstream out;
        out.unsetf(ios_base::right|ios_base::internal);
        return 0;],
        ac_cv_cxx_use_std_iostream=yes, ac_cv_cxx_use_std_iostream=no)

      if test "$ac_cv_cxx_use_std_iostream" = yes; then
			  ac_cv_cxx_have_std_iostream=yes
      fi
    fi

    AC_LANG_RESTORE
  ])

if test "$ac_cv_cxx_use_std_iostream" = yes; then
  AC_DEFINE(__USE_STD_IOSTREAM,,[define needed by some compilers to load new iostreams])
fi

if test "$ac_cv_cxx_have_std_iostream" = yes; then
  AC_DEFINE(HAVE_STD_IOSTREAM,,[define if the compiler supports the new standard iostream library])
fi
])

AC_DEFUN([AC_CXX_HAVE_STL],
[AC_CACHE_CHECK(whether the compiler supports Standard Template Library,
ac_cv_cxx_have_stl,
[AC_REQUIRE([AC_CXX_NAMESPACES])
 AC_LANG_SAVE
 AC_LANG_CPLUSPLUS
 AC_TRY_COMPILE([#include <list>
#include <deque>
#ifdef HAVE_NAMESPACES
using namespace std;
#endif],[list<int> x; x.push_back(5);
list<int>::iterator iter = x.begin(); if (iter != x.end()) ++iter; return 0;],
 ac_cv_cxx_have_stl=yes, ac_cv_cxx_have_stl=no)
 AC_LANG_RESTORE
])
if test "$ac_cv_cxx_have_stl" = yes; then
  AC_DEFINE(HAVE_STL,,[define if the compiler supports Standard Template Library])
else
  AC_DEFINE(__USE_STD_IOSTREAM,,[define necessary for the alpha native compilers])
  CMDSTD=-D_STANDARD_C_PLUS_PLUS
fi
])

AC_DEFUN([AC_CXX_TEMPLATES_EXPORT],
[

AC_CACHE_CHECK(wether the compiler supports the exporting of template definitions,
ac_cv_cxx_templates_export,
[AC_LANG_SAVE
 AC_LANG_CPLUSPLUS

 OLDFLAGS=$CXXFLAGS
 CXXFLAGS="$CXXFLAGS -Werror"

 AC_COMPILE_IFELSE(
   AC_LANG_SOURCE([
     template<class T> bool check(const T& t);
	   export template<class T> bool check(const T& t) { t != t; }]),
	 ac_cv_cxx_templates_export=yes, ac_cv_cxx_templates_export=no)
 
 CXXFLAGS=$OLDFLAGS
 AC_LANG_RESTORE
])

	
 if test "$ac_cv_cxx_templates_export" = yes; then
   AC_DEFINE(__EXPORT,export ,[define if the compiler supports template exporting])
	 COMPILE_TEMPLATES=yes;
 else
   AC_DEFINE(__EXPORT,,[define if the compiler supports template exporting])
	 AC_DEFINE(INCLUDE_TEMPL_DEFS,,[define if the compiler does not support template exporting])
	 COMPILE_TEMPLATES=no;
 fi
 AC_SUBST(COMPILE_TEMPLATES)
])

AC_DEFUN([AC_PROG_DOXYGEN],[
AC_REQUIRE([AC_EXEEXT])

AC_PATH_PROG(DOXYGEN, doxygen$EXEEXT, echo)
if test "$DOXYGEN" = echo; then
  AC_MSG_WARN([doxygen could not be found! Without doxygen you cannot generate the API documentation!])
fi;

AM_CONDITIONAL([CREATE_APIDOC], [test x$DOXYGEN != xecho])
AC_SUBST(DOXYGEN)

])

AC_DEFUN([AC_LIBRARY_VERSION],
[
	LIBRARY_VERSION=$1

  #retrieve the major minor and version step
  INTERFACE=`expr match "$LIBRARY_VERSION" ['\([0-9]*\):[0-9]*:[0-9]*']`			
	IMPLEMENTATION=`expr match "$LIBRARY_VERSION" ['[0-9]*:\([0-9]*\):[0-9]*']`			
	AGE=`expr match "$LIBRARY_VERSION" ['[0-9]*:[0-9]*:\([0-9]*\)']`			
 
	#calculate the libtoolize version system
	MAJOR=`expr $INTERFACE - $AGE`
	MINOR=`expr $INTERFACE - $MAJOR`
	VSTEP=$IMPLEMENTATION

	AC_SUBST(MAJOR)
	AC_SUBST(MINOR)
	AC_SUBST(VSTEP)
	AC_SUBST(LIBRARY_VERSION)

	AC_MSG_NOTICE("the library is: $MAJOR.$MINOR.$VSTEP")
])
