#ifndef BASEMAT_TMPL_METHODS
#define BASEMAT_TMPL_METHODS

// ==== TEMPLATE METHODS Basemat.cc ====

// 19-Apr-1999. Andras Aszodi

namespace RazorBack {

// ---- Scalar arithmetics ----

template<class T_>
Basemat_<T_>& Basemat_<T_>::operator*=(const T_& Factor)
{
    clone();
    for (register unsigned int i=0; i<eno(); i++)
	elems(i)*=Factor;
    return(*this);
}
// END of *=

template<class T_>
Basemat_<T_>& Basemat_<T_>::operator/=(const T_& Div)
{
    clone();
    for (register unsigned int i=0; i<eno(); i++)
	elems(i)/=Div;
    return(*this);
}
// END of /=

// ---- Output ----

// list_matrix: lists calling object to Out with entries occupying Width chars,
// in scientific format, Prec digits precision. If a row takes up 
// more than Linewidth chars, then the matrix is cut up nicely. 
// If Width<Prec+8,  then it is adjusted but no warning is given.
// NOTE: nobody knows what happens when T_ is not float or double :-)
template<class T_>
void Basemat_<T_>::list_matrix(ostream& Out, 
	unsigned int Prec, unsigned int Width, 
	unsigned int Linewidth) const
{
// NOTE: the "new" ISO C++ standard iostream library has different types
// and class structure than the "old" iostream. Until all the compilers
// migrate to this (notable exception is the buggy SGI "new" iostream),
// we provide a workaround as follows.
#ifdef HAVE_STD_IOSTREAM
#  define IOS_FLAGTYPE ios_base::fmtflags
#  define IOS_BASE ios_base
#else
#  define IOS_FLAGTYPE long
#  define IOS_BASE ios
#endif
    IOS_FLAGTYPE Oldflags=Out.flags();	// store previous settings here
    int Oldprec=Out.precision();
    unsigned int j,k,N, Cw, Rw, Chunk,Jbeg,Items,Ulinelen;

    if (Width<Prec+8) Width=Prec+8;	// adjust Width
    N=cno();
    Cw=prt_width(N);	// width of column index
    Rw=prt_width(rno());    // width of row index
    if (Cw>Width) Width=Cw;
    Items=(Linewidth-Rw-3)/(Width+1);	// columns per chunk

    // output will be right-justified by default
    Out.unsetf(IOS_BASE::right|IOS_BASE::internal);
    Out.setf(IOS_BASE::showpoint|IOS_BASE::scientific);	// 0.000000e+00 in C
    
    // main cycle: print chunks of the matrix   
    for (Jbeg=0,Chunk=(N-1)/Items+1; Chunk>0; Jbeg+=Items,Chunk--)
    {
	// underline length
	Ulinelen=(Chunk>1)? Items*(Width+1)+Rw+3:
		(N-Jbeg)*(Width+1)+Rw+3;
		
	// print column heading
	for (k=0; k<Rw+3; k++) Out.put(' ');  // leading spaces
	
	// print col indices, left-justified, Width width
	Out.setf(IOS_BASE::left);
	for (j=Jbeg; j<N && j<Jbeg+Items; j++)
	    Out << setw(Width) << j;
	Out << endl;
	
	// underline column heading
	for (k=0; k<Ulinelen; k++) Out.put('-');
	Out << endl;

	// print chunks for all rows: this varies for rectangular
	// and triangular matrices so a virt function call will do it
	print_rows(Out, Rw, Jbeg, Items, Width, Prec);

	// chunk separator ("---" in between, "===" at the end)
	for (k=0; k<Ulinelen; k++) Out.put(Chunk>1? '-': '=');
	Out << endl << endl;
    }
    
    // clean up mess
    Out.width(0);	    // each item uses its length as width
    Out.precision(Oldprec);	    // old precision
    Out.flags(Oldflags);    // reset flags to state before call
}
// END of list_matrix() 

} // RazorBack

// ==== END OF METHODS Basemat.cc ====

#endif	    // BASEMAT_TMPL_METHODS
