#ifndef FACTANAL_HEADER
#define FACTANAL_HEADER

// ==== HEADER Factanal.hh ====

/**
 * \file Factanal.hh
 * Factor analysis module.
 * The implementation is quite C-like, with global functions (no class).
 */

// 25-Aug-1998. Andras Aszodi

// ---- STANDARD HEADERS ----

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdlib.h>
#include <math.h>

#ifdef HAVE_STD
#  include <vector>
   using namespace std;
#else
#  include <vector.h>
#endif

// ---- MODULE HEADERS ----

#include "Symmat.hh"
#include "Rectmat.hh"

// ---- PROTOTYPES ----

namespace RazorBack {

/**
 * std_data(X, Mean, Sd): standardizes the data vector X
 * by calculating the mean and SD, and then subtracting the
 * mean from each entry and divide the centralised value by
 * the SD. Returns the original mean in Mean, the SD in Sd.
 * Return value: the no. of entries converted or 0 on error.
 */
unsigned int std_data(vector<double>& X, double& Mean, double& Sd);

/**
 * correl_data(X, Y): calculates the correlation coefficient
 * between the data vectors X and Y.
 * X and Y are assumed to have been standardised by
 * std_data() before the call.
 * Return value: the correlation coefficient (-1..+1).
 */
double correl_data(const vector<double>& X, const vector<double>& Y);

/**
 * calc_loadings(Correl, Load, Tol, Qual):
 * decomposes the correlation matrix Correl
 * so that the loadings will be returned in Load.
 * Return value: the number of factors which belong
 * to eigenvalues higher than Tol times the largest eigenvalue.
 * If Qual!=NULL, then a quality coefficient is returned in *Qual.
 */
int calc_loadings(const Symmat_<double>& Correl, Rectmat_<double>& Load,
        double Tol, double *Qual=NULL);

} // RazorBack

// ==== END OF HEADER Factanal.hh ====

#endif	// FACTANAL_HEADER
