// ==== METHODS MTqueue.cc ====
#include"MTqueue.hh"

// Classes that represent a "job queue" for multiple threads.
// A job object in the queue shall contain all information necessary
// for a thread to carry out its task.

// 15-May-2000, Andras Aszodi

// ==== STANDARD HEADERS ====

#ifdef HAVE_STD
#include <algorithm>
using namespace std;
#else
#include <algo.h>
#endif

// ==== MODULE HEADERS ====

using namespace RazorBack;

// ==== METHODS ====

// ---- Job queue manipulation ----

// pop_job():
// takes a job object wrapper from the priority queue,
// removes it from the queue and returns it.
// If the queue was empty then an empty wrapper is returned.
MTqueue_::Jobq_ MTqueue_::pop_job()
{
    Jobq_ Jq; // empty wrapper
    Qmutex.lock();
    if (!Jobqueue.empty())
    {
	Jq=Jobqueue.top();	// get wrapped job object pointer from queue
	Jobqueue.pop(); 	// throw entry away from queue
    }
    Qmutex.unlock();
    return Jq;
}
// END of pop_job()

// ==== GLOBAL FUNCTIONS ====

// MTqueue_start_workthread(Qptr):
// this is a global C function that must be executed by the
// "working threads" that process the jobs submitted to the queue.
// The working threads run indefinitely if not cancelled explicitly,
// so launch them via MTmgrdetach_ as detached threads.
// The MTqueue object supplying the jobs is pointed to by Qptr;
// type safety cannot be guaranteed here, be careful!
// Returns NULL when the thread finished.
namespace RazorBack {
void* MTqueue_start_workthread(void* Qptr)
{
extern Mutex_ Mtxdebug;
    MTqueue_* Qp=static_cast<MTqueue_*>(Qptr);
    MTqueue_::Jobq_ Jq;
    
    // almost infinite loop :-)
    while (1)
    {
	Qp->wait_4job();    // wait for a job pushed on the queue
	    	    	    // this is a cancellation point
	Jq=Qp->pop_job(); // get job pointer from priority queue
	if (!Jq.empty())   // there is work to do
	{
	    Jq.Jobptr->task();	    // perform the task
	    Jq.Doneptr->decrement_and_signal();   // tell producer that a job is done
	}
    }
    
    return NULL;    // no diagnostics
}
// END of MTqueue_start_workthread(Qptr)
} // Razorback

// ==== END OF METHODS MTqueue.cc ====
