AC_DEFUN([AC_CXX_HAVE_STD_IOSTREAM],
[AC_CACHE_CHECK(whether the compiler supports the new standard iostream library,
ac_cv_cxx_have_std_iostream,
[AC_REQUIRE([AC_CXX_NAMESPACES])
 AC_REQUIRE([AC_CXX_HAVE_STD])
 AC_LANG_SAVE
 AC_LANG_CPLUSPLUS
 AC_TRY_COMPILE([
#include <iostream>
#include <sstream>
#ifdef HAVE_NAMESPACES
using namespace std;
#endif],
[
ios_base::fmtflags flags_;
stringstream out;
out.unsetf(ios_base::right|ios_base::internal);
return 0;],
 ac_cv_cxx_have_std_iostream=yes, ac_cv_cxx_have_std_iostream=no)
 AC_LANG_RESTORE
])
if test "$ac_cv_cxx_have_std_iostream" = yes; then
  AC_DEFINE(HAVE_STD_IOSTREAM,,[define if the compiler supports the new standard iostream library])
fi
])


# Check for Razorback compiler flags, linker flags, and binary packages
AC_DEFUN([CHECK_RAZORBACK],
[
AC_REQUIRE([AC_PROG_CXX])

AC_ARG_WITH([RazorBack-Incl], 
						AC_HELP_STRING([--with-RazorBack-Incl=DIR],
	                         [the directory of the RazorBack library include files]),
						[RAZORBACK_INCL=$withval])
AC_ARG_WITH([RazorBack-Lib], 
						AC_HELP_STRING([--with-RazorBack-Lib=DIR],
	                         [the directory of the RazorBack library files]),
						[RAZORBACK_LIB=$withval])

AC_MSG_CHECKING([RazorBack include directory])
# Check that RAZORBACK_INCL is defined or that --with-RazorBack-Incl given
if test x"$RAZORBACK_INCL" = x ; then
RAZORBACK_INCLUDE_SEARCH="/usr/include/RazorBack /usr/include/razorback /usr/local/RazorBack /usr/local/razorback /usr/local/include/RazorBack /usr/local/include/razorback /usr/share/include/RazorBack /usr/share/include/razorback /opt/include/RazorBack /opt/include/razorback"

  for i in $RAZORBACK_INCLUDE_SEARCH; do
    if test -f $i/Vector.hh -a x"$RAZORBACK_INCL" = x; then 
      RAZORBACK_INCL=$i; 
    fi
  done
fi
if test x"$RAZORBACK_INCL" = x ; then
    AC_MSG_ERROR([*** RAZORBACK_INCL must be defined, or --with-RazorBack-Incl option given])
fi
AC_MSG_RESULT([$RAZORBACK_INCL])

AC_MSG_CHECKING([RazorBack library directory])
# Check that RAZORBACK_LIB is defined or that --with-RazorBack-Lib given
if test x"$RAZORBACK_LIB" = x ; then
  RAZORBACK_LIB_SEARCH="$libdir /usr/lib /usr/local/lib /usr/share/lib /opt/lib"

  for i in $RAZORBACK_LIB_SEARCH; do
    if test -f $i/libRazorBack.so -a x$RAZORBACK_LIB = x; then 
      RAZORBACK_LIB=$i; 
			break
    fi
    if test -f $i/libRazorBack.a -a x$RAZORBACK_LIB = x; then 
      RAZORBACK_LIB=$i; 
      break
    fi
  done
fi
if test x"$RAZORBACK_LIB" = x ; then
    AC_MSG_ERROR([*** RAZORBACK_LIB must be defined, or --with-RazorBack-Lib option given])
fi
AC_MSG_RESULT([$RAZORBACK_LIB])

])

AC_SUBST(RAZORBACK_INCL)
AC_SUBST(RAZORBACK_LIB)

