/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#include "common.hh"

#include <surfacens.hh>

#ifdef HAVE_NAMESPACES
using namespace std;
SURFACENS_USE
#endif 

double Surface::double_limit = 1e-3;

//******************************************************************************
void Surface::copyVector(const RbDblV& src , RbDblV& dest, double lastelement)
//******************************************************************************
{
	/*
	 * A failproof assignment of arbitrary vectors: 
	 *
	 * 1. the maximal common number of components is copied from position to
	 *    dest.  
	 * 2. if there are less components in position it the remaining components of
	 *    dest are filled with zeros except the last one which is set to 1.
	 */
	int cpycnt = MIN(src.dim(),dest.dim());
	int size   = dest.dim();
	int i;
	for(i=0;i<cpycnt;++i)
		dest[i] = src[i];
	
	for(i=cpycnt;i<size;++i)
		dest[i] = (i<(size-1)) ? 0 : lastelement;
}

//******************************************************************************
RbDblV Surface::cross(const RbDblV& a, const RbDblV& b)
//******************************************************************************
{
	RbDblV product(4,0);
	product[0] = a[1]*b[2] - a[2]*b[1];
	product[1] = a[2]*b[0] - a[0]*b[2];
	product[2] = a[0]*b[1] - a[1]*b[0];
	product[3] = 1.0;

	return product;
}
