/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#include "doublettchecker.hh"
#include "point.hh"

#ifdef HAVE_NAMESPACES
using namespace std;
using namespace Surface;
#endif 

//******************************************************************************
int DoublettChecker::operator()(const Point& point)
//******************************************************************************
{
	typedef multiset<spot_t>::const_iterator spotsit;
	typedef multiset<spot_t>::iterator spots_it;

	//reset the state
	wasdoublett_ = false;

	//create a new spot.
	spot_t newspot(-1, point);

	//get all the points that have the same radius
	spotsit begin = sSpots.lower_bound(newspot);
	spotsit end   = sSpots.upper_bound(newspot);

	while ((begin!=end) && (newspot.index==-1))
	{
		//if the spots are equal store the doublett index
		if (*begin==newspot) 
		{
			newspot.index = begin->index;
			wasdoublett_ = true;
		}
			++begin;
	}

	if (!wasdoublett_)	
	{
		//add the spot if it is unique with a new index
		newspot.index = sSpots.size();
		sSpots.insert(newspot);
	}

	return
		newspot.index;
}

//******************************************************************************
void DoublettChecker::clear()
//******************************************************************************
{
	sSpots.clear();
	wasdoublett_ = false;
}
