/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#include "fileformatfactory.hh"
#include "surfaceio.hh"

#ifdef HAVE_NAMESPACES
using namespace std;
using namespace Surface;
#endif 

//******************************************************************************
bool FileFormatFactory::registerClass(const std::string& name, format_t factory)
//******************************************************************************
{
  factories_cit f = factories_.find(name);
  if (f==factories_.end())
  {
    factories_.insert(make_pair(name,factory));
    return true;
  }
  else
    return false;
}

//******************************************************************************
bool FileFormatFactory::unregisterClass(const std::string& name)
//******************************************************************************
{
  factories_it f = factories_.find(name);
  if (f!=factories_.end())
  {
    factories_.erase(f);
    return true;
  }
  else
    return false;
}

//******************************************************************************
bool FileFormatFactory::hasClass(const std::string& name) const
//******************************************************************************
{
  factories_cit f = factories_.find(name);
  return f!=factories_.end();
}

//******************************************************************************
SurfaceIO* FileFormatFactory::createClass(const std::string& name) const
//******************************************************************************
{
  factories_cit f = factories_.find(name);
  if (f!=factories_.end())
    return f->second.method();
  else
    return 0;
}

//******************************************************************************
const string& FileFormatFactory::getDescription(const std::string& name) const
//******************************************************************************
{
  factories_cit f = factories_.find(name);
  if (f!=factories_.end())
    return f->second.description;
  else
    return 0;
}

//******************************************************************************
FileFormatFactory& FileFormatFactory::instance()
//******************************************************************************
{
	static FileFormatFactory instance;
	return instance;
}
