/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

/**
 * \file haspoint.hh
 *
 * Author : Christian Hofbauer (christian.hofbauer@pharma.novartis.com)\n
 * started at 08 Apr 2004, last CVS commit 2004/04/08 09:55:20
 */

#ifndef HASPOINT_HH
#define HASPOINT_HH

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifdef HAVE_STD
#  include <functional>
#else
#  include <function.h>
#endif

#include "triangle.hh"

#ifdef HAVE_NAMESPACES
namespace Surface
{
#endif 

	/**
	 * \brief a predicate that checks if a triangle includes a specific point
	 *
	 * The constructor of this predicat expects a point iterator as defined in
	 * the Triangle class. The function operator is called with a Triangle object
	 * as an argument and looks in its edge points if the query point can be
	 * found.
	 */
	template<class Triangle_>
	class HasPoint : public std::unary_function<const Triangle_&, bool>
	{
	public:
		/** 
		 * \brief standard constructor
		 * 
		 * \param point the query surface point iterator
		 */
		HasPoint(const Triangle::points_it& point) : point_(point) { }
		
		/** 
		 * \brief checks if the point is a part of the \p triangle 
		 * 
		 * \return \c true if the point is an edge of \p triangle, \c false
		 * otherwise.
		 */
		bool operator()(const Triangle_& triangle) {
			for(int i=A;i<=C;++i)
					if (triangle->getVertex(i) == point_) return true;
			return false;
		}
			
	private:

		/// \brief the surface point iterator
		Triangle::points_it point_;
	};

#ifdef HAVE_NAMESPACES
};
#endif 

#endif // HASPOINT_HH
