/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#include "point.hh"

#ifdef HAVE_NAMESPACES
using namespace std;
using namespace RazorBack;
using namespace Surface;
#endif

//**************************************************************************
Point::Point(const RbDblV& position, const RbDblV& normal, 
								 int master,int nearestatom, facetype_t facetype)
//**************************************************************************
: master_(master), nearestatom_(nearestatom),
	facetype_(facetype), position_(4, 0.0), normal_(4, 0.0)		
{
	setPosition(position);
	setNormal(position);
}

//******************************************************************************
Point& Point::operator=(const Point& src)
//******************************************************************************
{
  master_ = src.master_;
  nearestatom_ = src.nearestatom_;
  facetype_ = src.facetype_; 
  position_ = src.position_;
  normal_ = src.normal_;

  return * this;
}

//******************************************************************************
void Point::setPosition( const RbDblV& position )
//******************************************************************************
{
	copyVector(position, position_);
}

//******************************************************************************
void Point::setNormal(const RbDblV& normal)
//******************************************************************************
{
	copyVector(normal, normal_);
}

//****************************************************************************
void Point::setData(size_t col, double value)
//****************************************************************************
{
  if ((col<0) || (col>=data_->dim()))
    throw OutOfRange_e("The column index was out of bounds!");
				
  (*data_)[col] = value;
}
		
//******************************************************************************
void Point::setData(const RbDblV& pointdata)
//******************************************************************************
{
  copyVector(pointdata, *data_, 0);
}
