/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#include "surfaceview.hh"
#include "viewiterator.hh"

#ifdef HAVE_NAMESPACES
using namespace std;
using namespace Surface;
#endif 

//#include <iostream>

//******************************************************************************
SurfaceView::~SurfaceView() 
//******************************************************************************
{ 
  notify(); 

  if (surface_) 
    surface_->detach(*this); 

  if (entityowner_ && (surface_))
    delete surface_;
}

//******************************************************************************
void SurfaceView::notify()
//******************************************************************************
{
	for(list<ViewIterator*>::iterator i=observers_.begin();i!=observers_.end();)
	{
		list<ViewIterator*>::iterator observer = i++;
		(*observer)->update();
	}
}

//******************************************************************************
void SurfaceView::attach(ViewIterator& observer)
//******************************************************************************
{
	list<ViewIterator*>::iterator pos = find(observers_.begin(), observers_.end(),
																					&observer);
	if (pos == observers_.end())
	{
		observers_.push_back(&observer);
		observer.setSurfaceView(*this);
	}

}

//******************************************************************************
void SurfaceView::detach(ViewIterator& observer)
//******************************************************************************
{
	list<ViewIterator*>::iterator pos = find(observers_.begin(), observers_.end(),
																					&observer);
	
	if (pos != observers_.end())
		observers_.erase(pos);
}

//******************************************************************************
SurfaceView* SurfaceView::clone() const
//******************************************************************************
{
  //copy info
  DeepCopyInformation copyinfo;

  //entity
  SurfaceEntity* newentity = new SurfaceEntity(surface_->cloneTemplate());

  //view with new entity and ownership of entity
  copyinfo.newview = createView();
  copyinfo.newview->setSurfaceEntity(*newentity);
  copyinfo.newview->entityowner_ = true;

  buildSelectionEntity(copyinfo);
  buildSelectionView(copyinfo);
  
  postCopying(copyinfo);

  return copyinfo.newview;
}

//**************************************************************************
bool SurfaceView::setSurfaceEntity(SurfaceEntity& surface)
//**************************************************************************
{ 
  if (!surface_)
  {
    surface_ = &surface; 
    surface_->attach(*this);
    return true;
  }
  else
    return false;
}

//**************************************************************************
SurfaceEntity& SurfaceView::getSurfaceEntity() const
//**************************************************************************
{ 
  if (!surface_)
    throw InvalidReference_e("surface_ points to NULL");
  
  return *surface_; 
}

//**************************************************************************
SurfaceEntity* SurfaceView::takeSurfaceEntity() const
//**************************************************************************
{
  entityowner_ = false;
  return surface_;
}

