/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#include "triangle.hh"
#include "point.hh"

#ifdef HAVE_NAMESPACES
using namespace std;
using namespace Surface;
using namespace RazorBack;
#endif

//******************************************************************************
void Triangle::copyFlags(const Triangle& src)
//******************************************************************************
{
  facetype_ = src.facetype_;
  facenum_  = src.facenum_;
}

//******************************************************************************
Triangle::points_it& Triangle::operator[](int ix)
//******************************************************************************
{
	if ((ix>=0) && (ix<3))
		return vertices_[ix];
	else
	{
		throw OutOfRange_e("The index of a Triangle::operator[] call was outside [0..2]!");
	}
}

//******************************************************************************
Triangle::points_it Triangle::successor(points_it vertex) const
//******************************************************************************
{
	int i=0;
	while ((i<3) && (vertex!=vertices_[i])) ++i;
	
	if (i<3)
		return vertices_[(i==2) ? 0 : i+1];
	else
		return 0;
}

//******************************************************************************
RbDblV Triangle::getCenter() const
//******************************************************************************
{
	RbDblV center(4,0.0);
	center[W] = 1.0;

	for(int i=X;i<W;++i)
	{
		for(int j=A;j<=C;++j)
			center[i] += vertices_[j]->getPosition()[i];
		center[i] /= 3;
	}

	return center;	
}

//******************************************************************************
RbDblV Triangle::getNormal() const
//******************************************************************************
{
	RbDblV normal = calcNormal();
	normal.vec_norm();
	return normal;
}

//******************************************************************************
double Triangle::getArea() const
//******************************************************************************
{
	return calcNormal().vec_len()/2.0;
}

//******************************************************************************
RbDblV Triangle::calcNormal() const
//******************************************************************************
{
	RbDblV normal(3,0);
	copyVector(cross(vertices_[C]->getPosition()-
									 vertices_[B]->getPosition(),
									 vertices_[A]->getPosition()-
									 vertices_[B]->getPosition()), normal);
	return normal;
}
