/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

/**
 * \file viewbuilder.hh
 *
 * Author : Christian Hofbauer (christian.hofbauer@pharma.novartis.com)\n
 * started at 18 Mar 2004, last CVS commit 2004/03/18 13:36:14
 */

#ifndef VIEWBUILDER_HH
#define VIEWBUILDER_HH

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifdef HAVE_STD
#else
#endif

//#include <iostream>


#ifdef HAVE_NAMESPACES
namespace Surface
{
#endif 
	class SurfaceEntity;
	class SurfaceView;

	/**
	 * \brief abstract base class for all view builders
	 *
	 * The task of the abstract view builder is the definition of the view
	 * builder interface and the management of the SurfaceEntity pointer
	 * (surface_). The view builder interface consists of three methods:
	 * initialize() to start with the building process and select, to select()
	 * all the necessary points and triangles. Finally the ready SurfaceView
	 * object will be returned by getView()
	 */
	class ViewBuilder
	{
	public:

		//**************************************************************************
		// CONSTRUCTOR
		//**************************************************************************

		/**
		 * \brief default constructor
		 *
		 * It sets the pointer to the SurfaceEntity to NULL.
		 */
		ViewBuilder() : surface_(0) { }

		/**
		 * \brief initializes the viewer creation process
		 *
		 * A call to initialize() must be the first statement when working with a
		 * ViewBuilder. This should create (in the concrete subclasses) the
		 * SurfaceView and make a copy of the given \p surface reference.
		 *
		 * \param surface a reference to the SurfaceEntity
		 *
		 * \sa select()
		 */
		virtual void initialize(SurfaceEntity& surface) { surface_=&surface; }
		/**
		 * \brief performs the selection of points and triangles
		 *
		 * The select method builds the view object. It selects the necessary
		 * points and triangles and constructs the view according to its internal
		 * representation and type.
		 */
		virtual void select() = 0;
		
		/**
		 * \brief retrieves the ready SurfaceView
		 *
		 * \return a pointer to the constructed and ready SurfaceView.
		 */
		virtual SurfaceView* getView() const = 0;

	protected:
		
		/**
		 * \brief a pointer to the corresponding SurfaceEntity
		 */
		SurfaceEntity* surface_;
	};

#ifdef HAVE_NAMESPACES
};
#endif 

#endif // VIEWBUILDER_HH
