/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#include "doublett_test.hh"

#include <float.h>
#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include "templates.hh"
#include "utils.hh"

#include <surface.hh>
#include <offio.hh>

#ifdef HAVE_STD
#  include <iostream>
#else
#  include <iostream.h>
#endif

#ifdef HAVE_NAMESPACES
using namespace boost::unit_test_framework;
using namespace Surface;
using namespace std;
#endif 

//******************************************************************************
void doublett_test::test_doubletts()
//******************************************************************************
{
	BOOST_MESSAGE( "\n" << DELIMITER << "\n Test disabled doublett checking\n");

	OffIO offio(false);

	BOOST_MESSAGE(" !! load doubletts.off");
	SurfaceEntity* doubletts = offio.read("doubletts.off");

	BOOST_MESSAGE(" ** check for 1132 points and 2260 triangles\n");
	BOOST_CHECK(doubletts->getNPoints() == 1132);
	BOOST_CHECK(doubletts->getNTriangles() == 2260);

	delete doubletts;
}
			
//******************************************************************************
void doublett_test::test_nodoubletts()
//******************************************************************************
{
	BOOST_MESSAGE( "\n" << DELIMITER << "\n Test enabled doublett checking\n");

	Surface::double_limit = 1e-3;
	OffIO offio(true);

	BOOST_MESSAGE(" !! load doubletts.off");
	SurfaceEntity* doubletts = offio.read("doubletts.off");
	
	BOOST_MESSAGE(" ** check for 1131 points and 2258 triangles\n");
	BOOST_CHECK(doubletts->getNPoints() == 1131);
	BOOST_CHECK(doubletts->getNTriangles() == 2258);

	BOOST_MESSAGE(" !! load nodoubletts.off (reference)");
	SurfaceEntity* nodoubletts = offio.read("nodoubletts.off");	

	check_surfaces(*doubletts, *nodoubletts);
}

//******************************************************************************
doublett_test_suite::doublett_test_suite() 
//******************************************************************************
: test_suite("doublett_test_suite")
{
	// add member function test cases to a test suite
	boost::shared_ptr<doublett_test> instance(new doublett_test);
		
	test_case* doubletts_test_case = 
		BOOST_CLASS_TEST_CASE(&doublett_test::test_doubletts,	instance); 
	test_case* nodoubletts_test_case = 
		BOOST_CLASS_TEST_CASE(&doublett_test::test_nodoubletts,	instance); 

	//test case, number of tests that should fail
	add(doubletts_test_case, 0);
	add(nodoubletts_test_case, 0);
}
