/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#include "templates.hh"

#include <common.hh>

#ifdef HAVE_NAMESPACES
using namespace Surface;
#endif 

double dodek_points[20][3] = {
	{  1.214124,  0.000000,  1.589309 },
	{	 0.375185,  1.154701,  1.589309 },
	{ -0.982247,  0.713644,  1.589309 },
	{ -0.982247, -0.713644,  1.589309 },
	{  0.375185, -1.154701,  1.589309 },
	{  1.964494,  0.000000,  0.375185 },
	{  0.607062,  1.868345,  0.375185 },
	{ -1.589309,  1.154701,  0.375185 },
	{ -1.589309, -1.154701,  0.375185 },
	{  0.607062, -1.868345,  0.375185 },
	{  1.589309,  1.154701, -0.375185 },
	{ -0.607062,  1.868345, -0.375185 },
	{ -1.964494,  0.000000, -0.375185 },
	{ -0.607062, -1.868345, -0.375185 },
	{  1.589309, -1.154701, -0.375185 },
	{  0.982247,  0.713644, -1.589309 },
	{ -0.375185,  1.154701, -1.589309 },
	{ -1.214124,  0.000000, -1.589309 },
	{ -0.375185, -1.154701, -1.589309 },
	{  0.982247, -0.713644, -1.589309 }
};

double dodek_normals[20][3] = {
	{  0.607,   -0.000,    0.795 },
	{  0.188,    0.578,    0.795 },
	{ -0.491,    0.357,    0.795 },
	{ -0.491,   -0.357,    0.795 },
	{  0.188,   -0.578,    0.795 },
	{  0.982,   -0.000,    0.188 },
	{  0.304,    0.934,    0.188 },
	{ -0.795,    0.578,    0.188 },
	{ -0.795,   -0.578,    0.188 },
	{  0.304,   -0.934,    0.188 },
	{  0.795,    0.578,   -0.188 },
	{ -0.304,    0.934,   -0.188 },
	{ -0.982,   -0.000,   -0.188 },
	{ -0.304,   -0.934,   -0.188 },
	{  0.795,   -0.578,   -0.188 },
	{  0.491,    0.357,   -0.795 },
	{ -0.188,    0.578,   -0.795 },
	{ -0.607,    0.000,   -0.795 },
	{ -0.188,   -0.578,   -0.795 },
	{  0.491,   -0.357,   -0.795 }
};

int dodek_connections[36][3] = {
	{  0,  1,  2, },
	{  1,  7,  2, },
	{  2,  7, 12, },
	{	17, 12,  7, },
	{ 18, 12, 17, },
	{ 19, 18, 17, },
	{ 19, 13, 18, },
	{ 19,  9, 13, },
	{  3, 13,  9, },
	{  3,  9,  4, },
	{  0,  3,  4, },
	{  4,  5,  0, },
	{  0,  5, 10, },
	{	15, 10,  5, },
	{ 16, 10, 15, },
	{ 19, 16, 15, },
	{ 19, 17, 16, },
	{  0,  2,  3, },
	{  2,  8,  3, },
	{  3,  8, 13, },
	{	18, 13,  8, },
	{  0, 10,  6, },
	{ 16,  6, 10, },
	{	16, 11,  6, },
	{ 17, 11, 16, },
	{	17,  7, 11, },
	{  0,  6,  1, },  
	{  1,  6, 11, },
	{  1, 11,  7, },
	{  2, 12,  8, },
	{ 18,  8, 12, },
	{  4,  9, 14, },
	{	19, 14,  9, },
	{ 15, 14, 19, },
	{  4, 14,  5, },
	{ 15,  5, 14, }
};

double dodek_onsurfacedists[20] = 
{ 
	0, 
	1.42729, 
	2.30940, 
	2.30940, 
	1.42729, 
	1.42729, 
	2.30940, 
	3.73669, 
	3.73669, 
	2.85458, 
	2.30940, 
	3.73669, 
	4.61880, 
	4.28187, 
	2.85458, 
	3.73669, 
	4.61880, 
	6.04609,
	5.70915, 
	4.28187
};


int dodek_edges = 36 * 3;

//******************************************************************************
void makeDodekaeder(Surface::SurfaceEntity& surface,
										Surface::facetype_t facetype, int facenum)
//******************************************************************************
{


	SurfaceEntity::points_it pits[20];
	int i;

	surface.setPointDensity(1.0);
	surface.setProbeRadius(1.5);
	surface.setSphereCount(1);
	
	//points
	for(i=0;i<20;++i)
	{
		pits[i] = surface.addPoint();
		pits[i]->setPosition(RbDblV(dodek_points[i], 3));
		pits[i]->setNormal(RbDblV(dodek_normals[i], 3));
		pits[i]->setMasterIndex(i);
		pits[i]->setFaceType(facetype);
		pits[i]->setNearestAtom(0);
	}
 
	//triangles
	for(i=0;i<36;++i)
	{
		SurfaceEntity::triangles_it t = 
			surface.addTriangle(pits[dodek_connections[i][0]], 
													pits[dodek_connections[i][1]], 
													pits[dodek_connections[i][2]]);

		t->setFaceType(facetype);
		t->setFaceNum(facenum);
	}
}
