/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#include "triangle_test.hh"

#include <float.h>
#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include "templates.hh"
#include "utils.hh"

#include <surface.hh>

#ifdef HAVE_STD
#  include <iostream>
#else
#  include <iostream.h>
#endif

#ifdef HAVE_NAMESPACES
using namespace boost::unit_test_framework;
using namespace Surface;
using namespace std;
#endif 

const double DODECAEDER_AREA = 42.0585;

const double FIRST_CENTER[3] = { 0.202, 0.623, 1.589 };

const double FIRST_NORMAL[3] = { 0, 0, 1.937334 };
const double FIRST_NORMAL_LENGTH = 1.937334;

//******************************************************************************
triangle_test::triangle_test()
//******************************************************************************
: surface()
{
	makeDodekaeder(surface);
}

//******************************************************************************
void triangle_test::test_center()
//******************************************************************************
{
	BOOST_MESSAGE("\n" << DELIMITER << "\n Triangle center test \n");

	BOOST_MESSAGE(" ** calculate and check the center of the first triangel");
	RbDblV center = surface.beginTriangles()->getCenter();
	for (int i=X;i<W;++i)
		BOOST_CHECK_CLOSE(center[i], FIRST_CENTER[i], 1);
	
}
			
//******************************************************************************
void triangle_test::test_normal()
//******************************************************************************
{
	BOOST_MESSAGE("\n" << DELIMITER << "\n Triangle normal test \n");

	RbDblV normal = surface.beginTriangles()->getNormal();
	for (int i=X;i<W;++i)
		BOOST_CHECK_CLOSE(normal[i], FIRST_NORMAL[i]/FIRST_NORMAL_LENGTH, 1);
	
}

//******************************************************************************
void triangle_test::test_area()
//******************************************************************************
{
	BOOST_MESSAGE("\n" << DELIMITER << "\n Triangle area test \n");

	BOOST_MESSAGE(" ** calculate and check the area of the first triangle");
	BOOST_CHECK_CLOSE(surface.beginTriangles()->getArea(), FIRST_NORMAL_LENGTH/2.0, 
										1e-2);

	BOOST_MESSAGE(" ** calculate and check the area of the dodecaeder");
	double area = 0.0;
	for(SurfaceEntity::triangles_it t = surface.beginTriangles();
			t!=surface.endTriangles();++t)
		area += t->getArea();
	
	BOOST_CHECK_CLOSE(area, DODECAEDER_AREA, 1.0e-2);
}

//******************************************************************************
triangle_test_suite::triangle_test_suite() 
//******************************************************************************
: test_suite("triangle_test_suite")
{
	// add member function test cases to a test suite
	boost::shared_ptr<triangle_test> instance(new triangle_test);
		
	test_case* center_test_case = 
		BOOST_CLASS_TEST_CASE(&triangle_test::test_center,	instance); 
	test_case* normal_test_case = 
		BOOST_CLASS_TEST_CASE(&triangle_test::test_normal,	instance); 
	test_case* area_test_case = 
		BOOST_CLASS_TEST_CASE(&triangle_test::test_area,	instance); 
		
	//test case, number of tests that should fail
	add(center_test_case, 0);
	add(normal_test_case, 0);
	add(area_test_case, 0);
}
