/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#include <float.h>
#include <unistd.h>

//BOOST.test
#include <boost/test/included/unit_test_framework.hpp>
#include <boost/test/unit_test_log.hpp>

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifdef HAVE_STD
#  include <iostream>
#else
#  include <iostream.h>
#endif

#ifdef HAVE_NAMESPACES
using boost::unit_test_framework::test_suite;
using boost::unit_test_framework::test_case;
using boost::unit_test_framework::unit_test_log;

using namespace std;
#endif 

#include "utils.hh"
#include "templates.hh"
#include "doublett_test.hh"
#include "patchview_test.hh"
#include "off_test.hh"
#include "msms_test.hh"
#include "sld_test.hh"
#include "surface_test.hh"
#include "triangle_test.hh"
#include "vectorview_test.hh"
#include "graphview_test.hh"

//******************************************************************************
void test_sized()
//******************************************************************************
{
	RbDblV posvector(4);
	
	cout << DELIMITER << endl
			 << "Information section " << endl
			 << DELIMITER << endl
			 << endl;
	cout << "size of Point                   : " 
			 << sizeof(Surface::Point) + sizeof(posvector) * 2 << endl;
	cout << "size of Triangle                : " 
			 << sizeof(Surface::Triangle) << endl;
	cout << "size of entity point    iterator: " << 
		sizeof(Surface::SurfaceEntity::points_it) << endl;
	cout << "size of entity triangle iterator: " << 
		sizeof(Surface::SurfaceEntity::triangles_it) << endl;
	
}

//******************************************************************************
void usage(std::ostream& out = cout)
//******************************************************************************
{
  out << "Options:\n"
			<< "  -s           print the sizes of different libsurface datatypes\n"
			<< "  -v           verbose output use often for higher verbosity (up to 10 times)\n"
			<< "  -h           displays this screen\n"
			<< endl;	
}

//******************************************************************************
bool getOptions(int argc, char* argv[], bool sizes)
//******************************************************************************
{
  extern char *optarg;
  extern int optind;
  
  bool error_flag = false;

  int c;
  while ((c = getopt(argc, argv, "hs")) != -1)
  {
    char buffer[200];
    switch(c)
    {
      case 'h':
      {
				usage(std::cout);
				break;
      }

      case 's':
      {
				sizes = true;
				break;
      }

      case '?':
				error_flag = true;
    }
  }

  if (error_flag) 
    usage(std::cerr);

	return !error_flag;
}

//******************************************************************************
test_suite* init_unit_test_suite(int argc, char* argv[]) 
//******************************************************************************
{
	int verbose = 10;
	bool sizes = false;

	if (getOptions(argc, argv, sizes))
	{
		
		if (sizes) test_sized();
		
		std::auto_ptr<test_suite> test( BOOST_TEST_SUITE( "libsurface tests" ) );

		test->add(new surface_test_suite());
		test->add(new triangle_test_suite());
		test->add(new vectorview_test_suite());
		test->add(new graphview_test_suite());
		test->add(new patchview_test_suite());
		test->add(new msms_test_suite());
		test->add(new off_test_suite());
		test->add(new sld_test_suite());
		test->add(new doublett_test_suite());
	
		return test.release();
	}
	else
		return (test_suite*)0;
}
