/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

/**
 * \file atom.hh
 *
 * Author : Christian Hofbauer (christian.hofbauer@pharma.novartis.com)\n
 * started at 20 Jul 2004, last CVS commit 2004/07/20 14:00:58
 */

#ifndef ATOM_HH
#define ATOM_HH

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifdef HAVE_STD
#include <list>
#else
#include <list.h>
#endif

#include <common.hh>

/**
 * \brief a very compact atom structure
 * 
 * The atom structure is needed for atomic field calcutions. It contains a
 * position vector and a fieldprop value that is used to build the molecular
 * field.
 */
struct Atom
{
	/** 
	 * \brief default constructor
	 */
	Atom(double x, double y, double z, double prop)
		: pos(4,0), fieldprop(0) {
		pos[0] = x; pos[1] = y; pos[2] = z; pos[3] = 1; fieldprop = prop; }
	Atom() : pos(4,0), fieldprop(0) { pos[3] = 1; }
	
	/// \brief the atom's position in 3D space
	RbDblV pos;
	/// \brief the property value that generates the field
	double fieldprop;
};

typedef list<Atom> atoms_t;
typedef atoms_t::iterator atoms_it;
typedef atoms_t::const_iterator atoms_cit;

#endif // ATOM_HH
