/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#include "dlloader.hh"

#include <stdlib.h>
#include <dlfcn.h>

#ifdef HAVE_STD
#  include <iostream>
#else
#  include <iostream.h>
#endif

#include <boost/tokenizer.hpp>

#ifdef HAVE_NAMESPACES
using namespace std;
using namespace boost;
#endif 

//******************************************************************************
DLLoader::DLLoader(const string& key)
//******************************************************************************
: handles_()
{
  typedef boost::tokenizer<boost::char_separator<char> > tokenizer;

  char*  lib_string = getenv(key.c_str());
  if (lib_string!=0)
  {
    string libraries = lib_string;

    char_separator<char> sep(":");
    tokenizer tokens(libraries, sep);
    for(tokenizer::iterator t=tokens.begin(); t!=tokens.end();++t)
    {
      void* handle = dlopen(t->c_str(),RTLD_NOW);
      if (!handle)
        cerr << "Could not load the dynamic library " << *t << endl;
      else
        handles_.push_back(handle);
    }
  }
}

//******************************************************************************
DLLoader::~DLLoader()
//******************************************************************************
{
  for(list<void*>::const_iterator h=handles_.begin();h!=handles_.end();++h)
    dlclose(*h);
}
