/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#include "fieldcalculator.hh"

#ifdef HAVE_STD
#  include <fstream>
#else
#  include <fstream.h>
#endif


#ifdef HAVE_NAMESPACES
using namespace std;
SURFACENS_USE
#endif 

//******************************************************************************
bool FieldCalculator::setParameter(const std::string& name, 
																	 const std::string& value)
//******************************************************************************
{
	if (uppercase(name) == "PDB")
	{
		if (!readMolecule(value.c_str()))
			throw WrongParameterValue(name, value);
		return true;
	}
	else
		return false;
}

//******************************************************************************
bool FieldCalculator::readMolecule(const char* filename)
//******************************************************************************
{
	ifstream pdbfile(filename);

	char line[128];
	
	if (!pdbfile)
		return false;

	while (!pdbfile.eof())
	{
		pdbfile.getline(line, 128);
		
		char record[7] = "      ";
		sscanf(line, "%6s", &record);


		if ((strncmp(record,"ATOM", 4)==0) || (strncmp(record, "HETATM", 6)==0))
		{
			char dummy[30];
			double x, y, z, o;

			if (!sscanf(line, "%30c%lf %lf %lf %lf", &dummy, &x, &y, &z, &o))
				throw PDBReadError(filename);
			atoms_.push_back(Atom(x, y, z, o));
		}
	}
	
	pdbfile.close();

	return true;
}

