/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#include "globalcurvselector.hh"

#include <simplebuilder.hh>
#include <spherebuilder.hh>
#include <graphview.hh>

#ifdef HAVE_NAMESPACES
using namespace std;
SURFACENS_USE
#endif 

/**
 * In their paper Zachmann et. al. apply a correction factor of 0.97 to the
 * on-surface distances calculated by the Dijkstra algorithm. 
 */
const double DISTANCE_CORRECTION = 0.97;


//******************************************************************************
GlobalCurvSelector::GlobalCurvSelector(SurfaceEntity* surface, double radius)
//******************************************************************************
:CurvSelector(surface), radius_(radius), graph_(0)
{
  SimpleBuilder<GraphView> builder;
  graph_ = dynamic_cast<GraphView*>(surface_->createView(builder));
}

//******************************************************************************
GlobalCurvSelector::~GlobalCurvSelector()
//******************************************************************************
{
	if (graph_) delete graph_;
}

//******************************************************************************
size_t GlobalCurvSelector::operator()(const SurfaceEntity::points_it& p)
//******************************************************************************

{
	points_.clear();

  graph_->compOnSurfaceDistances(p);
  for(GraphView::vertexIter v=graph_->begin();v!=graph_->end();++v)
    if ((graph_->getDistance(v)*DISTANCE_CORRECTION)>radius_)
    {
      GraphView::vertexIter w = graph_->getPredecessor(v);
      if ((graph_->getDistance(w)*DISTANCE_CORRECTION)<radius_)
      {
        points_.insert(v);
        points_.insert(w);        
      }
    }

	selection_it c=points_.find(p);
	if (c!=points_.end()) points_.erase(c);

  return points_.size();
}
