/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

/**
 * \file globalcurvselector.hh
 *
 * Author : Christian Hofbauer (christian.hofbauer@pharma.novartis.com)\n
 * started at 19 Jul 2004, last CVS commit 2004/07/19 07:31:29
 */

#ifndef GLOBALCURVSELECTOR_HH
#define GLOBALCURVSELECTOR_HH

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "curvselector.hh"

#include <surfacens.hh>
SURFACE_FORWARD(GraphView);

/**
 * \brief a point selector for global curvature calculation
 * 
 * The selector inherits from CurvSelector and selects the points for a global
 * curvature calculation. A global curvature calculation does not use the
 * immedeate neighoborhood of the surface point but all the points which are
 * placed around a ring of a given radius away from the central point (see also
 * operator()()).
 */
class GlobalCurvSelector : public CurvSelector
{
public:

	//****************************************************************************
	// PUBLIC METHODS
	//****************************************************************************

	/** 
	 * \brief standard constructor
	 *
	 * The constructor also calculates a proper preselection distance.
	 * 
	 * \param surface a pointer to the handled surface
	 */
	GlobalCurvSelector(SURFACENS SurfaceEntity* surface, double radius);
	~GlobalCurvSelector();

	/** 
	 * \brief selection operator
	 *
	 * The selection operator selects the reference point and all points that are
	 * placed around the selection radius. That is it selects the points just off
	 * the cutoff plus their immedeate predecessors, determined during a run of
	 * Dijkstra's algorithm.
	 *
	 * \param p the current reference point that serves as the center of the
	 *          calculation.
	 * 
	 * \return the number of selected points
	 */
  virtual size_t operator()(const SURFACENS SurfaceEntity::points_it& p);

private:
	
	/// \brief the cutoff radius of the selection
	double radius_;

private:

	//@{
	/// \brief internal elements used by the algorithm
	SURFACENS GraphView* graph_;
  SURFACENS GraphView* buildView(const SURFACENS SurfaceEntity::points_it& p);
	//@}


};

#endif // GLOBALCURVSELECTOR_HH
