/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#include "localcurvselector.hh"

#include <simplebuilder.hh>
#include <graphview.hh>

#ifdef HAVE_NAMESPACES
using namespace std;
SURFACENS_USE
#endif 

//******************************************************************************
LocalCurvSelector::LocalCurvSelector(SurfaceEntity* surface)
	: CurvSelector(surface), graph_(0)
//******************************************************************************
{
	SimpleBuilder<GraphView> builder;
	graph_ = dynamic_cast<GraphView*>(surface_->createView(builder));
}

//******************************************************************************
LocalCurvSelector::~LocalCurvSelector()
//******************************************************************************
{ 
	if (graph_) delete graph_; 
}

//******************************************************************************
size_t LocalCurvSelector::operator()(const SurfaceEntity::points_it& p)
//******************************************************************************
{
	if (graph_)
	{
		points_.clear();

		GraphView::adjacencyIter b,e;
		boost::tie(b,e) = graph_->adjacency(graph_->selectionOf(p));

		while (b!=e) 
			points_.insert(b++);
		
		return points_.size();
	}
	else
		return 0;
}
