/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

/**
 * \file localcurvselector.hh
 *
 * Author : Christian Hofbauer (christian.hofbauer@pharma.novartis.com)\n
 * started at 19 Jul 2004, last CVS commit 2004/07/19 07:31:29
 */

#ifndef LOCALCURVSELECTOR_HH
#define LOCALCURVSELECTOR_HH

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifdef HAVE_STD
#else
#endif

#include "curvselector.hh"

#include <surfacens.hh>
SURFACE_FORWARD(GraphView);

/**
 * \brief a point selector for local curvature calculation
 * 
 * The selector inherits from CurvSelector and selects the points for a local
 * curvature calculation. A local curvature calculation uses the adjacent
 * neighoborhood of the surface point to calculate the fitting paraboloid (see
 * also operator()()).
 */
class LocalCurvSelector : public CurvSelector
{
public:
	
	//****************************************************************************
	// PUBLIC METHODS
	//****************************************************************************

	/** 
	 * \brief standard constructor
	 * 
	 * \param surface a pointer to the handled surface
	 */
	LocalCurvSelector(SURFACENS SurfaceEntity* surface);

	/// \brief destructor
	~LocalCurvSelector();

	/** 
	 * \brief selection operator
	 *
	 * The selection operator selects the ajacent neighborhood of the given
	 * point.
	 *
	 * \param p the current reference point that serves as the center of the
	 *          calculation.
	 * 
	 * \return the number of selected points
	 */
  virtual size_t operator()(const SURFACENS SurfaceEntity::points_it& p);

private:
	
	/// \brief internal graph view for point selection
	SURFACENS GraphView* graph_;

};

#endif // LOCALCURVSELECTOR_HH
