/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

/**
 * \file primitives.hh
 *
 * Author : Christian Hofbauer (christian.hofbauer@pharma.novartis.com)\n
 * started at 06 Jul 2004, last CVS commit 2004/07/06 12:10:19
 */

#ifndef PRIMITIVES_HH
#define PRIMITIVES_HH

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifdef HAVE_STD
#  include <list>
#  include <iostream>
#else
#  include <list.h>
#  include <iostream.h>
#endif

#include <common.hh>

#ifdef HAVE_NAMESPACES
namespace Surface
{
#endif 

	class VectorView;
	class Triangle;
	class SurfaceEntity;

	class Primitive
	{
	public:
		Primitive(const Triangle& triangle, 
							double innerdielectric = 1.0, double outerdielectric = 78.390);

		Primitive(const Primitive& src);
		Primitive& operator=(const Primitive& src);

		const RbDblV& getCenter() const { return center_; }
		const RbDblV& getNormal() const { return normal_; }
		double getArea() const { return area_; }

		double getInnerDielectric() const { return innerdielectric_; }
		double getOuterDielectric() const { return outerdielectric_; }
		int    getAtom() const { return atom_; }

	private:

		RbDblV center_;
		RbDblV normal_;
		double area_;
		
		double innerdielectric_;
		double outerdielectric_;
		int    atom_;
	};

	class Primitives
	{

		//**************************************************************************
		// PRIVATE TYPE DEFINITIONS
  	//**************************************************************************

		typedef list<Primitive> primitives_t;
		typedef primitives_t::iterator primitives_it;
		typedef primitives_t::const_iterator primitives_cit;

	public:

		//**************************************************************************
		// PUBLIC TYPE DEFINITONS
		//**************************************************************************
		
		typedef primitives_cit primitiveIter;

		//****************************************************************************
		// CONSTRUCTOR AND DESTRUCTOR
		//****************************************************************************

		Primitives(SurfaceEntity& surface, 
							 double innerdielectric = 1.0, double outerdielectric = 78.390);
		~Primitives();

		//****************************************************************************
		// PUBLIC METHODS
		//****************************************************************************
		void init(SurfaceEntity& surface, 
							double innerdielectric = 1.0, double outerdielectric = 78.390);

		void operator()();

		bool write(const std::string& filename);
		bool write(std::ostream& stream);
		
	private:

		//****************************************************************************
		// PRIVATE MEMBERS
		//****************************************************************************

	
		double innerdielectric_;
		double outerdielectric_;

		VectorView* view_;
		primitives_t primitives_;
	};


#ifdef HAVE_NAMESPACES
};
#endif 

#endif // PRIMITIVES_HH
