/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#include "propcalcfactory.hh"
#include "propcalculator.hh"

#ifdef HAVE_NAMESPACES
using namespace std;
#endif 

//******************************************************************************
bool PropCalcFactory::registerClass(const std::string& name, factory_t factory)
//******************************************************************************
{
  factories_cit f = factories_.find(name);
  if (f==factories_.end())
  {
    factories_[name] = factory;
    return true;
  }
  else
    return false;
}

//******************************************************************************
bool PropCalcFactory::unregisterClass(const std::string& name)
//******************************************************************************
{
  factories_it f = factories_.find(name);
  if (f!=factories_.end())
  {
    factories_.erase(f);
    return true;
  }
  else
    return false;
}

//******************************************************************************
bool PropCalcFactory::hasClass(const std::string& name) const
//******************************************************************************
{
  factories_cit f = factories_.find(name);
  return f!=factories_.end();
}

//******************************************************************************
PropCalculator* PropCalcFactory::createClass(const std::string& name) const
//******************************************************************************
{
  factories_cit f = factories_.find(name);
  if (f!=factories_.end())
    return f->second();
  else
    return 0;
}

//******************************************************************************
PropCalcFactory& PropCalcFactory::instance()
//******************************************************************************
{
  if (!instance_)
    instance_ = new PropCalcFactory();
  return *instance_;
}

PropCalcFactory* PropCalcFactory::instance_ = 0;
