/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

/**
 * \file propgenerator.hh
 *
 * Author : Christian Hofbauer (christian.hofbauer@pharma.novartis.com)\n
 * started at 20 Jul 2004, last CVS commit 2004/07/20 13:37:11
 */

#ifndef PROPGENERATOR_HH
#define PROPGENERATOR_HH

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifdef HAVE_STD
#  include <list>
#  include <map>
#  include <iostream>
#else
#  include <list.h>
#  include <map.h>
#  include <iostream.h>
#endif

#include <string>
#include <surfacens.hh>
SURFACE_FORWARD(SurfaceEntity);
SURFACE_FORWARD(SurfaceIO);

class PropCalculator;

class PropGenerator 
{
public:
	
	PropGenerator();
	~PropGenerator();
	bool init(int argc, char** argv);

	bool operator()();

	static void usage(std::ostream& out);

	bool wantHelp() const { return help_; }
	int  getStatus() const { return status_; }
	
private:

	//****************************************************************************
	// PRIVATE TYPES
	//****************************************************************************

	typedef std::map<std::string, std::string> parameters_t;
	typedef parameters_t::iterator parameters_it;
	typedef parameters_t::const_iterator parameters_cit;

	typedef std::list<PropCalculator*> props_t;
	typedef props_t::iterator props_it;
	typedef props_t::const_iterator props_cit;

	bool addPropCalculator(const std::string& id);

	bool parseOptions(int argc, char** argv);
	bool loadFormat(const std::string& format, bool checkdoubletts=false);
	bool readParameter(const char* specifier);

	bool readSurface();
	bool writeSurface();
	void writeData(std::ostream& out);
	
	props_t properties_;
	parameters_t parameters_;

	std::string surfacefile_;
	SURFACENS SurfaceIO* surfaceio_;
	std::string outputfile_;
	
	bool rewrite_;
	bool help_;

	int status_;

	SURFACENS SurfaceEntity* surface_;
};


#endif // PROPGENERATOR_HH
