/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#include "assocgraph.hh"

//*******************************************************************************
void writegraph(ostream& stream, assocgraph_t& graph, const char* title)
//*******************************************************************************
{
	assocgraph_traits::indexmap_t indexmap = get(vertex_index, graph);
	assocgraph_traits::pairmap_t pairmap = get(vertex_pair, graph);
	assocgraph_traits::vertices_t vits = vertices(graph);
	assocgraph_traits::edges_t eits = edges(graph);

	
	stream << "digraph " << title << "{" << endl;

	//vertices
	int ix=0;
	for(assocgraph_traits::vertex_it v=vits.first;v!=vits.second;++v)
	{
		indexmap[*v] = ix++;
		stream << "\t" << indexmap[*v];
		stream << " [ label=\"" 
					 << pairmap[*v].first << "-" << pairmap[*v].second
					 << "\" ]\n";
	}
	
	stream << endl;
	
	//edges
	for(assocgraph_traits::edge_it e=eits.first;e!=eits.second;++e)
	{
		int source_ix = indexmap[source(*e, graph)];
		int target_ix = indexmap[target(*e, graph)];
		
		stream << "\t" << source_ix << "->" << target_ix << "\n";
	}

	//graph closing bracket
	stream << "}" << endl;
	
}

//*******************************************************************************
assocgraph_traits::vertex_t createVertex(assocgraph_t& graph)
//*******************************************************************************
{
	assocgraph_traits::rfuncmap_t rfuncmap = get(vertex_Rfunc, graph);

	assocgraph_traits::vertex_t newv = add_vertex(graph);
	put(rfuncmap, newv, (double*)0);

	return newv;
}

//*******************************************************************************
void removeVertex(assocgraph_traits::vertex_t v, assocgraph_t& graph)
//*******************************************************************************
{
	assocgraph_traits::rfuncmap_t rfuncmap = get(vertex_Rfunc, graph);
	
	double* rfunc = rfuncmap[v];
	if (rfunc) delete[] rfunc;

	clear_vertex(v, graph);
	remove_vertex(v, graph);
}

//*******************************************************************************
void clearGraph(assocgraph_t& graph)
//*******************************************************************************
{
	assocgraph_traits::rfuncmap_t rfuncmap = get(vertex_Rfunc, graph);

	assocgraph_traits::vertices_t vits = vertices(graph);
	
	for(assocgraph_traits::vertex_it vertex=vits.first;vertex!=vits.second;)
	{
		assocgraph_traits::vertex_t v = *vertex++;
		double* rfunc = rfuncmap[v];
		if (rfunc) delete[] rfunc;
		
		clear_vertex(v, graph);
		remove_vertex(v, graph);
	}
}
