/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#include "correspondent.hh"

#include <Vector.hh>
#include <math.h>

//*******************************************************************************
Vector_<double> rotate(const Vector_<double>& point, const double angle)
//*******************************************************************************
{
	double sin_a = sin(angle);
	double cos_a = cos(angle);

	Vector_<double> result(3,0);
	
	//rotate the vertices

	result[0] = point[0] * cos_a - point[1] * sin_a;
	result[1] = point[0] * sin_a + point[1] * cos_a;

	return result;
}

//*******************************************************************************
size_t Correspondent::addCorrespondencies(Trimesh& mapA, Trimesh& mapB, 
																					const double angle)
//*******************************************************************************
{
	typedef list<Vector_<double> >::const_iterator pos_it;

	list<Vector_<double> > posA, posB;
	Trimesh::vertex_it v,w;

	//copy the rotated positions of vertices in A
	for(v=mapA.begin();v!=mapA.end();++v)
		posA.push_back(rotate(mapA.vertCoordinates(*v), angle));

	//copy the positions of vertices in B
	for(w=mapB.begin();w!=mapB.end();++w)
		posB.push_back(mapB.vertCoordinates(*w));

	pos_it p = posA.begin();
	for(v=mapA.begin();v!=mapA.end();++v, ++p)
	{
		pos_it q = posB.begin();
		for(w=mapB.begin();w!=mapB.end();++w, ++q)
		{
			Trimesh::vertex_t V = mapA.vertMasterIx(*v);
			Trimesh::vertex_t W = mapB.vertMasterIx(*w);
			
			lCorresponds.push_back(correspond_t(V,W, (*p - *q).vec_len()));
		}
	}

	lCorresponds.sort();
}
