/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#ifndef CORRESPONDENT_TEMPL_CC
#define CORRESPONDENT_TEMPL_CC

#ifdef HAVE_CONFIG_H
#  include <config.h>
#  ifdef HAVE_STD
#    include <algorithm>
#  else
#    include <algorithm.h>
#  endif //HAVE_STD
#else //HAVE_CONFIG_H
#  include <algorithm>
#endif //HAVE_CONFIG_H

//*******************************************************************************
template<class In> 
size_t Correspondent::addCorrespondencies(In begin, In end)
//*******************************************************************************
{
	corresponds_it pos = lCorresponds.begin();
	int c;
	
	while (begin!=end)
	{
		if (pos==lCorresponds.end())
		{
			//if we are at the end of lCorresponds
		  lCorresponds.insert(pos, *begin++);
			++c;
		}
		else
			//check if the actual position in the input sequence is smaller then the 
			if (*begin<*pos)
			{
				lCorresponds.insert(pos, *begin++);
				++c;
			}
			else
				++pos;
	}

	return c;
}

//*******************************************************************************
template<class Out>	
size_t Correspondent::operator()(Out ins)
//*******************************************************************************
{
	//initialization
	corresponds_it pos = lCorresponds.begin();
	int c = 0;

	/* LOOP: find correspondents */
	while (pos!=lCorresponds.end())
	{
		//add the closest correspondence pair to the output sequence
		*ins = *pos;
		
		//store values of closest point for comparison later in the code
		Trimesh::vertex_t insA = pos->first;
		Trimesh::vertex_t insB = pos->second;

		//remove all elements in the list related to either pos->A or pos->B.
		do
		{
			corresponds_it i = pos++;
			if ((i->first == insA) || (i->second == insB)) lCorresponds.erase(i);
		}
		while (pos!=lCorresponds.end());
				 
		pos = lCorresponds.begin();
		//increment the input iterator to receive the next position
		++ins;
		++c;
	}

	return c;
}


#endif //CORRESPONDENT_TEMPL_CC
