/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#include "scorecollection.hh"

#ifdef HAVE_NAMESPACES
using namespace std;
#endif

#include "scorer.hh"
#include "cluster.hh"


//*******************************************************************************
ScoreCollection::~ScoreCollection()
//*******************************************************************************
{
	clear();
}

//*******************************************************************************
void ScoreCollection::clear()
//*******************************************************************************
{
	//delete all the Scorer objects
	for(scorerlist_it s=lScorer.begin();s!=lScorer.end();++s)
		delete *s;
	
	//delete the list entry
	lScorer.clear();
}
	
//*******************************************************************************
void ScoreCollection::operator()(Cluster* cluster) const
//*******************************************************************************
{
	//score with all Scorer objects
	for(scorerlist_cit s=lScorer.begin();s!=lScorer.end();++s)	
		(*(*s))(cluster);
}

//*******************************************************************************
ScoreCollection::scorerlist_it ScoreCollection::addScorer(Scorer* const scorer)
//*******************************************************************************
{
	return lScorer.insert(lScorer.end(),scorer);
}

//*******************************************************************************
bool ScoreCollection::deleteScorer(scorerlist_it it)
//*******************************************************************************
{
	size_t oldsize = lScorer.size();
	lScorer.erase(it);
	return oldsize>lScorer.size();
}

//*******************************************************************************
bool ScoreCollection::deleteScorer(Scorer* const scorer)
//*******************************************************************************
{
	size_t oldsize = lScorer.size();
	lScorer.remove(scorer);
	return oldsize>lScorer.size();	
}

