/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#ifndef UTILS_HH
#define UTILS_HH

#include <mysql.h>

#include <Rectmat.hh>

#include <trimesh.hh>
#include <patch.hh>

#ifdef HAVE_CONFIG_H
#  include <config.h>
#  ifdef HAVE_STD
#    include <iostream>
#    include <string>
#    include <vector>
#  else
#    include <iostream.h>
#    include <string.h>
#    include <vector.h>
#  endif //HAVE_STD
#else //HAVE_CONFIG_H
#  include <iostream>
#  include <string>
#  include <vector>
#endif //HAVE_CONFIG_H

#ifdef HAVE_NAMESPACES
namespace RazorBack
{
#endif
	class Distrapprox_;
#ifdef HAVE_NAMESPACES
};
#endif

#include <Vector.hh>

/*
	The experiment_t is the set of experimental parameters needed for the task
 */

struct experiment_t
{
	string name;
	
	int surface_a;
	int surface_b;

	int center_a;
	int center_b;

	double radius;
	double rotation_step;
	int img_resolution;

	int cols;

	Rectmat_<double> key1_data;
	Rectmat_<double> key2_data;

	vector<string> colnames[2];

	experiment_t():
			surface_a(0), surface_b(0), radius(0.0), rotation_step(0.0),
			img_resolution(0), cols(0), key1_data(), key2_data() { }
			
};

struct clusterdb_t
{
	int id;
	int experiment;
	double tolerance;
	double cutoff;
	double flimit;
	double cluster;
	int patch_a;
	int patch_b;
	double R;
	double RMS;
};

//database login data

#define DB_USER "hofbach1"
#define DB_HOST "puffin"
#define SURFS_TABLE "surfaces"
#define PATCH_TABLE "patches"
#define EXPERIMENTS_TABLE "experiments"
#define CENTERS_TABLE "centers"
#define TB_EXPERIMENTS "experiments"
#define TB_CLUSTERS "clusters"

//*******************************************************************************
// database reading and writing functions
//*******************************************************************************

Trimesh* readSurf_DB(MYSQL* con, const int surface_id,
										 double& probe, double& density);

Vector_<double> readCenter_DB(MYSQL* con, const int center_id);

int writeSurf_DB(MYSQL* con, Trimesh* trimesh,
								 const double probe, const double density, const int level);

Trimesh* readPatch_DB(MYSQL* con, 
											const int surface_id, const int center, const double r);
Trimesh* readPatch_DB(MYSQL* con, const int patch_id);

int writePatch_DB(MYSQL* con, surfacePatch* patch, 
									const int parent, const double radius, 
									const int center);

bool readExperiment_DB(MYSQL* con, experiment_t & exp, 
											 const int exp_id, const bool allcols=false);
//*******************************************************************************
// DATA files reader functions
//*******************************************************************************

int read_header(istream& stream, vector<string>& columns);
int read_data(string file_name, int cols, Rectmat_<double>& data,
							vector<string>& columns);


//*******************************************************************************
// STATISTIC FUNCTIONS
//*******************************************************************************

template<class In, class Out>
pair<Out,Out> calcQuartils(In begin, const int size);
template<class It>
pair<double, double> standardize(It begin, It end);

//*******************************************************************************
// CONVERSION FUNCTIONS
//*******************************************************************************

string ftoa(const double value, const int precision = 0);
string itoa(const int value);


#ifdef INCLUDE_TMPL_DEFS
  #include "utils_templ.cc"
#endif


#endif //UTILS_HH
