/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#ifndef UTILS_TEMPL_CC
#define UTILS_TEMPL_CC

//*******************************************************************************
template<class In, class Out>
pair<Out,Out> calcQuartils(In begin, const int size)
//*******************************************************************************
{
	double pos1 = (size-1)*0.25;
	double pos3 = (size-1)*0.75;
	int i;
	pair<Out,Out> quartils;
	
	for(i=1;i<floor(pos1);++i)	++begin;

	quartils.first = *begin;
	if (pos1>floor(pos1))
	{
		quartils.first += *(++begin);
		quartils.first /=2;
		pos1 = ceil(pos1);
	}

	for(i=pos1;i<floor(pos3);++i) ++begin;

	quartils.second = *begin;
	if (pos3>floor(pos3))
	{
		quartils.second += *(++begin);
		quartils.second /=2;
	}
		
	return quartils;
}

//*******************************************************************************
template<class It>
pair<double, double> standardize(It begin, It end)
//*******************************************************************************
{
	double mean = 0.0;
	double stdev = 0.0;
	
	int n=0;
	It i;

	//mean +stddev
	for(i=begin;i!=end;++i) 
	{
		mean += *i;
		stdev += (*i) * (*i);
		++n;
	}
	mean /= n;
	stdev /=n;

	stdev -= mean*mean;
	stdev = sqrt(stdev);

	//standardize to mean=0 stdev=1
	for(i=begin;i!=end;++i) 
	  *i = (*i-mean)/stdev;

	return make_pair(mean, stdev);
}

#endif
