.TH "surfcomp" 3 "12 Jul 2004" "Version 1.0.0" "surfcomp" \" -*- nroff -*-
.SH NAME 
surfcomp \- molecular surface comparison program

.SH SYNOPSIS 
.BI "surfconvert [ --help -v[v] ] -h|--host " "DATABASE_HOST"
.BI "-d|--database "   "DATABASE_NAME"
.BI "-o|--output "     "OUTPUT_PREFIX"
.BI "-e|--experiment " "EXPERIMENT_NUMBER"
.BI "--rlevel "        "R"
.BI "--mindist "       "MINIMAL_DISTANCE"
.BI "--tolerance "     "DISTANCE_TOLERANCE"
.BI "--angle "         "ANGLE_TOLERANCE"
.BI "--fuzzytol "      "FUZZY_TOLERANCE"
.BI "--cpcutoff "      "CRITICAL_POINT_CUTOFF"
.BI "--cpcol "         "CP_COLUMN"
.BI "--fuzzycol "      "FUZZY_COLUMN"
.BI "--duckcol "       "HSI_COLUMN"
.BI "--linkage "       "single|complete|average"

.SH DESCRIPTION

Analysis of the distributions of physicochemical properties mapped onto
molecular surfaces can highlight important similarities or differences between
compound classes, contributing to rational drug design efforts. 
.B surfcomp
presents an approach that uses maximal common subgraph comparison and harmonic
shape image matching to detect locally similar regions between two molecular
surfaces augmented with properties such as the electrostatic potential or
lipophilicity. The complexity of the problem is reduced by a set of filters
that implement various geometric and physicochemical heuristics. A paper that
describes the background details of the program has been published:
.PP
Hofbauer C., Lohninger H., Aszodi A.,
.I "SURFCOMP: A Novel Graph-Based Approach to Molecular Surface Comparison"
, J. Chem. Inf. Comp. Sci,
.B 44:3
, 837-847, 2004.
.PP
The program takes the surface data from an existing mysql database and the
augmenting data from a data file. The fundamental parameters which will not
change for different comparisons of the same surface pair are retrieved from a
database table while the most varying parameters (i.e. the heuristic filter
values) are passed to the program as options.
.PP 
The result of the program is always and XML file that contains the different
detected clusters as a tree that has emerged from the clustering algorithm. If
the
.B -v
option has been given twice the program also produces a lot of intermediate
output that reports the state of the association graph after each filtering
step.

.SH OPTIONS

Default values are given in brackets.

.IP --help
print a short help message
.IP -v
increases thelevel of verbosity (two levels are supported)
.IP -h|--host
the computer name of the database hast (the address MySQL server)
.IP -d|--database
the name of the database where the surfaces and experiments are stored
.IP -o|--output 
the base name of all output files. The name of the result file will be
.B <OUTPUT>.xml
and the intermediate files (if any) will have names like
.B <OUTPUT>_*.*
.
.IP -e|--experiment
The number of the experiment in the chosen database. Note that database ids
usually start with 1.
.IP --rlevel [ = 0.0 ]
The filter threshold of the harmonic shape image (HSI) filter. It corresponds
to the minimal pearson correlation coefficient between two patches that is
allowed for matching pairs.
.IP --mindist [ = 0.5 ]
The minimal distance between two related patch pairs
.IP --tolerance [ = 0.5 ]
The maximal tolerated difference of the on surface distances between two
related patch pairs on both surfaces.
.IP --angle [ = 15.0 ]
The maximal tolerated angle difference in the overlap filtering. This angle
corresponds to the difference between the headings from one patch pair to the
other on both surfaces
.IP --fuzzytol [ = 0.0 ]
The threshold of the fuzzy property filter. It represents the maximal
dissimilarity between two related pairs that passes the fuzzy filter.
.IP --cpcutoff [ = 2.0 ]
The cutoff value for the calculation of the critical points. A critical point
is defined as a point that has a maximal or minimal principle curvature among
all its neighbors within this cutoff value.
.IP --cpcol [ = 0 ]
The mask that selects the data columns for the critical point detection
.IP --fuzzycol [ = 0 ]
The mask that selects the data column for the fuzzy point filtering
.IP --duckcol [ = 0 ]
The mask that selects the data column for the HSI filtering (the name is not
very illustrative, but it is a reference to the development code name of this
program).
.IP --linkage [ = single ]
specifies the kind of linkage in the final clustering. Possible variations are 
.I single
, 
.I complete
and
.I average
linkage.

.SS Data columns
Data columns are specified with a bit mask. Thus a datafile can only have 32
columns, but this will be enough for even the most ambitous surface comparison
experiments. If a filter needs one or more data column these have to be
specified with an integer value where every 1, in the binary sequence, means
that this column has been selected for that filter:

e.g. 
.I FUZZY_COLUMN 
= 8 means that the fourth column was selected for the fuzzy
filter.

or 
.I CP_COLUMN
= 12 means that the third and fourth column have been selected for the critical
point determination.

.SH DIAGNOSTICS

.SH AUTHOR
Christian Hofbauer <christian.hofbauer@pharma.novartis.com>
