/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#include "filter.hh"
#include <string.h>

//*******************************************************************************
FilterError::FilterError(const int code, const char* msg)
//*******************************************************************************
:fCode(code), fMessage(0)
{
	fMessage = new char[strlen(msg)+1];
	strcpy(fMessage,msg);
}
	
//*******************************************************************************
FilterError::~FilterError()
//*******************************************************************************
{
	if (fMessage)	delete[] fMessage;
}	

//*******************************************************************************
Filter::Filter()
//*******************************************************************************
{
	pDistances[0] = pDistances[1] = 0;
	pSurfaces[0] = pSurfaces[1] = 0;
}

//*******************************************************************************
Filter::Filter(Trimesh** surfaces, distancemap_t** distances)
//*******************************************************************************
{
	pDistances[0] = pDistances[1] = 0;
	pSurfaces[0] = pSurfaces[1] = 0;

	init(surfaces, distances);
}
	
//*******************************************************************************
void Filter::init(Trimesh** surfaces, distancemap_t** distances)
//*******************************************************************************
{
	for(int i=0;i<2;++i)
	{
		pDistances[i] = distances[i];
		pSurfaces[i] = surfaces[i];
	}
}

//*******************************************************************************
size_t Filter::removeIsolated(assocgraph_t& assocgraph, const int valence)
//*******************************************************************************
{
	int c = 0;

	assocgraph_traits::vertices_t vits = vertices(assocgraph);
	for(assocgraph_traits::vertex_it v=vits.first;v!=vits.second;)
	{
		//remove all vertices with outdegree(v) <= valence
		assocgraph_traits::vertex_it w = v++;
		if (out_degree(*w, assocgraph)<=valence) 
		{
			removeVertex(*w, assocgraph);
			++c;
		}
	}
	
	return c;
}
