/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#include "positionfilter.hh"

#ifdef HAVE_CONFIG_H
#  include <config.h>
#  ifdef HAVE_STD
#    include <map>
#  else
#    include <map.h>
#  endif
#else
#  include <map>
#endif 

//*******************************************************************************
PositionFilter::PositionFilter(Trimesh** surfaces, distancemap_t** distances, 
															 const Vector_<double>& sphereA, 
															 const Vector_<double>& sphereB)
//*******************************************************************************
: Filter(surfaces, distances)
{
	init(surfaces, distances, sphereA, sphereB);
}

//*******************************************************************************
void PositionFilter::init(Trimesh** surfaces, distancemap_t** distances, 
													const Vector_<double>& sphereA, 
													const Vector_<double>& sphereB)
//*******************************************************************************
{
	Filter::init(surfaces, distances);

	fCenter[0].dim(3);
	fCenter[1].dim(3);
	for(int i=0;i<3;++i)
	{
		fCenter[0][i] = sphereA[i];
		fCenter[1][i] = sphereB[i];
	}

	fRadius[0] = sphereA[3];
	fRadius[1] = sphereB[3];
}

//*******************************************************************************
size_t PositionFilter::operator()(assocgraph_t& assocgraph)
//*******************************************************************************
{
	typedef map<Trimesh::vertex_t, double>::iterator distances_it;

	/* INITIALIZATION */
	int c=0;
	map<Trimesh::vertex_t, double> distances[2]; 

	//necessary property maps
	assocgraph_traits::pairmap_t pairmap = get(vertex_pair, assocgraph);
	
	assocgraph_traits::vertices_t vits = vertices(assocgraph);
	
//  	cout << endl;

	for(assocgraph_traits::vertex_it vit=vits.first;vit!=vits.second;)
	{
		//get the association graph vertex and the patch centers on the surfaces
		assocgraph_traits::vertex_it v=vit++;

		bool keep = true;
		int i = 0;

		Trimesh::vertex_t point[2];
		point[0] = pairmap[*v].first;
		point[1] = pairmap[*v].second;

		while ((i<2) && keep)
		{
			//look for the point in the distance map
			//if it was not found calculate the distance to the center of the surface
			double distance = 0;
			distances_it d = distances[i].find(point[i]);
			if (d==distances[i].end())
			{
				const Vector_<double>& pos = pSurfaces[i]->vertCoordinates(point[i]);
				distances[i][point[i]] = distance = (pos - fCenter[i]).vec_len();

// 				cout << "Pos " << point[i] <<" : " 
// 						 << pos[0] << ", " << pos[1] << ", " << pos[2] << ", " << endl;
// 				cout << "Center " << i << " : " 
// 						 << fCenter[i][0] << ", " << fCenter[i][1] << ", " << fCenter[i][2] << ", " << endl;

// 				cout << "Distace = " << distance << endl;
			}
			else
				distance = d->second;

			keep = (distance<=fRadius[i]);
			
// 			if (!keep)
// 				cout << i << " :  " << point[i] << " => " << distance << endl;

			++i;
		}

		if (!keep)
		{
			//delete the vertex pair 
			removeVertex(*v, assocgraph);
			++c;
		}
	}

	return c;
}

	
