# AC_CXX_BOOST_GRAPH
#-------------------------------------------------------------------------------
#Checks for the boost library directory
AC_DEFUN([AC_CXX_BOOST], [

AC_REQUIRE([AC_PROG_CXX])

AC_ARG_WITH([BOOST], 
						AC_HELP_STRING([--with-BOOST=DIR],
	                         [the directory of the BOOST libraries
	                         (def.=/usr/lib/boost/)]),
						[BOOSTDIR=$withval])

AC_MSG_CHECKING([BOOST library directory])
# Check that SURFACELIB_INCL is defined or that --with-SurfaceLib-Incl given
if test x"$BOOSTDIR" = x ; then
BOOSTDIR_SEARCH="/usr/lib/boost /usr/local/lib/boost /usr/share/lib/boost usr/share/lib/usr/include /usr/local/include /usr/share/include"

  for i in $BOOSTDIR_SEARCH; do
    if test -f $i/boost/any.hpp -a x"$BOOSTDIR" = x; then 
      BOOSTDIR=$i; 
    fi
  done
fi
if test x"$BOOSTDIR" = x ; then
    AC_MSG_ERROR([*** BOOSTDIR must be defined, or --with-BOOST option given])
fi
AC_MSG_RESULT([$BOOSTDIR])

AC_SUBST(BOOSTDIR)

])

# AC_CXX_BOOST_GRAPH
#-------------------------------------------------------------------------------
#Checks for the boost graph library
AC_DEFUN([AC_CXX_BOOST_GRAPH], [

	AC_REQUIRE([AC_CXX_BOOST])
	
	#check wether the graph library works properly
	AC_CACHE_CHECK(wether the BOOST graph library can be compiled and linked,
	ac_cv_cxx_boost_graph,
	[
		AC_LANG_PUSH(C++)

		OLDFLAGS=$CXXFLAGS
		CXXFLAGS="$CXXFLAGS -I$BOOSTDIR -D__USE_STD_IOSTREAM"
		if test -n "$BOOSTCONFIG"; then
			CXXFLAGS="$CXXFLAGS -I$BOOSTCONFIG"
		fi
			

		AC_LINK_IFELSE(
			AC_LANG_SOURCE([[
				#include <boost/concept_archetype.hpp>
				#include <boost/graph/dijkstra_shortest_paths.hpp>
				#include <boost/graph/graph_archetypes.hpp>

				typedef boost::default_constructible_archetype<
				  boost::sgi_assignable_archetype<> > dist_value;

				// So generate_infinity works...
				namespace std 
				{
				  template <> class numeric_limits<dist_value> 
					{
		  			public:
				  	  static dist_value max() 
							{ return boost::static_object<dist_value>::get(); }
				  };
				}

				dist_value abs(const dist_value& x) { return x; }
				std::size_t abs(std::size_t x) { return x; }

				int main()
				{
				  using namespace boost;
				  typedef default_constructible_archetype< 
				    sgi_assignable_archetype<equality_comparable_archetype<> > > vertex_t;

			    typedef incidence_graph_archetype<vertex_t, directed_tag, 
			      allow_parallel_edge_tag> IncidenceGraph;
			    typedef vertex_list_graph_archetype<vertex_t, directed_tag, 
			      allow_parallel_edge_tag, IncidenceGraph> graph_t;
			    graph_t& g = static_object<graph_t>::get();
			    vertex_t s;
			    typedef graph_traits<graph_t>::edge_descriptor edge_t;
			    readable_property_map_archetype<edge_t, std::size_t> weight;
			    readable_property_map_archetype<vertex_t, int> index;
			    read_write_property_map_archetype<vertex_t, std::size_t> distance;
			    dijkstra_shortest_paths(g, s, vertex_index_map(index).
                            			weight_map(weight).
                            			distance_map(distance));
				  return 0;
				}]]),
			ac_cv_cxx_boost_graph=yes,ac_cv_cxx_boost_graph=no)

		CXXFLAGS=$OLDFLAGS

  	AC_LANG_POP])

		if test "$ac_cv_cxx_boost_graph" = no; then
			AC_MSG_FAILURE([can't compile and link the BOOST graph lib. Make sure that the libary is found in $BOOSTDIR],
									  1)
		fi
])	

