/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#include <trimesh.hh>
#include <fileformats.hh>
#include <unistd.h>

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <cstdio>

#ifdef HAVE_STD
#  include <iostream>
#  include <fstream>
#else
#  include <iostream.h>
#  include <fstream.h>
#endif


//******************************************************************************
void usage()
//******************************************************************************
{
  cout << "Trimesh library - file conversion tool v 0.1 (c)NFI vienna 2002 \n" 
       << endl
       << "usage : convert_file [-i <input> -o <ouput> -d] input_format output_format\n"
       << endl
       << "The input is read from standard input or the file <input> if specified,\n"
       << "output is written to standard output or to the file <output>.\n" << endl; 
 
  cout << "Options:\n"
       << "  -i <input>   read the data from the file <input>\n"
       << "  -o <output>  write the data to the file <output>\n"
       << "               Extentions are set according to the specifies output file format!\n"
       << "  -d           remove doubletts in the input file (could cause large runtimes)\n"
       << "  -v           verbose output (recommended only with data output into a file)\n"
       << "  -h           displays this screen\n"
       << endl;

  cout << " Possible formats as arguments: ";
  for (int i=0;i<N_FILEFORMATS;++i)
    cout << trimesh_formatnames[i] << " ";
  cout << "\n" << endl;
 
}

//******************************************************************************
int main(int argc, char* argv[])
//******************************************************************************
{
  extern char *optarg;
  extern int optind;
  
  string input_file;   //filename of the input file
  string output_file; //filename of the output file

  trimesh_format in_format = msms_format;   //input format
  trimesh_format out_format = own_format;  //output format

  bool error_flag = false;

  //option flags
  bool removedoubletts = false;
  bool verbose         = false;
  bool help            = false;

  int c;
  while ((c = getopt(argc, argv, "o:i:dvh")) != -1)
  {
    char buffer[200];
    switch(c)
    {
      case 'o' :
      {
				if (!sscanf(optarg, "%s", buffer))
				{
					error_flag = true;
					break;
				}
	
				output_file = buffer;
				break;
      }

      case 'i' :
      {
				if (!sscanf(optarg, "%s", buffer))
				{
					error_flag = true;
					break;
				}
	
				input_file = buffer;
				break;
      }
      
      case 'd':
      {
				removedoubletts = true;
				break;
      }

      case 'v':
      {
				verbose = true;
				break;
      }

      case 'h':
      {
				help = true;
				break;
      }

      case '?':
				error_flag = true;
    }
  }
  if (error_flag) 
  {
    usage();
    return 1;
  }

  if (help)
  {
    usage();
    return 0;
  }

//read format arguments

  if (optind<argc)
  {
    string argument = argv[optind++];
    for(int i=0;i<N_FILEFORMATS;++i)
      if (argument==trimesh_formatnames[i]) 
      {
				in_format = (trimesh_format) i;
				break;
      }
  }
  
  if (optind<argc)
  {
    string argument = argv[optind++];
    for(int i=0;i<N_FILEFORMATS;++i)
      if (argument==trimesh_formatnames[i]) 
      {
				out_format = (trimesh_format) i;
				break;
      }
  }
  
  Trimesh* mesh;
  
  //reading the mesh

	mesh = readMesh(input_file,in_format,removedoubletts);
	
  if (!mesh)
  {
		if (input_file=="") input_file="stdin"; 
    cerr << "Error reading the input from " << input_file << "!"<< endl;
    return 1;
  }

  //writing the mesh

  bool result = writeMesh(mesh,output_file,out_format);

  if (!result)
  {
		if (output_file=="") output_file="stdout"; 
    cerr << "Error writing the output to " << output_file << endl;
    delete mesh;
    return 2;
  }
  
  delete mesh;
  return 0;
}
