/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#include <getopt.h>
#include <unistd.h>
#include <sys/types.h>
#include <unistd.h>
#include <iostream>
#include <fstream>
#include <cstdio>

#include <trimesh.hh>
#include <fileformats.hh>
#include <sldtrimesh.hh>

#include <Rectmat.hh>


//******************************************************************************
void usage()
//******************************************************************************
{
  cout << "Trimesh library - SLD tool v 0.1 (c)NFI vienna 2002 \n" 
       << endl
       << "usage : sldtool -i SLDFILE -o SURFACEFILE -d DATAFILE OUTPUTFORMAT" << endl;

  cout << " Possible output formats are: ";
  for (int i=0;i<N_FILEFORMATS;++i)
    cout << trimesh_formatnames[i] << " ";
  cout << "\n" << endl;
}

//*******************************************************************************
bool getOptions(int argc, char* argv[], 
								int& help, 
								string& input_file, string& output_file, string& data_file,
								trimesh_format& out_format)
//*******************************************************************************
{
	extern char *optarg;
  extern int optind;
  int c;

	static struct option long_options[] =
		{
			{"help",       no_argument,       &help,       1 },
			{"datafile",   required_argument, 0,          'd'},
			{"input",      required_argument, 0,          'i'},
			{"output",     required_argument, 0,          'o'},
			{0, 0, 0, 0}
		};
  int option_index = 0;       
	
  bool error_flag = false;

  while (1)
	{
		char buffer[200];
		float number = 0;
				
		c = getopt_long(argc, argv, "hi:o:d:",
										long_options, &option_index);

		if (c==-1) break;
		// read all options
    switch(c)
    {
      case 'o' :
      {
				if (!sscanf(optarg, "%s", buffer))
				{
					error_flag = true;
					break;
				}
	
				output_file = buffer;
				break;
      }

      case 'i' :
      {
				if (!sscanf(optarg, "%s", buffer))
				{
					error_flag = true;
					break;
				}
	
				input_file = buffer;
				break;
      }
      
      case 'd':
      {
				if (!sscanf(optarg, "%s", buffer))
				{
					error_flag = true;
					break;
				}
	
				data_file = buffer;
				break;
      }

      case 'h':
      {
				help = true;
				break;
      }

      case '?':
				error_flag = true;
    }
  }
	
//read format arguments

  if (optind<argc)
  {
    string argument = argv[optind++];
    for(int i=0;i<N_FILEFORMATS;++i)
      if (argument==trimesh_formatnames[i]) 
      {
				out_format = (trimesh_format) i;
				break;
      }
  }
  
	return !error_flag;
}

//******************************************************************************
int main(int argc, char* argv[])
//******************************************************************************
{
	/* initialization */

  string input_file;   //filename of the input file
  string output_file;  //filename of the output file
	string data_file;    //filename for the data file

  trimesh_format out_format = own_format;  //output format

  int help = 0;

	if (!getOptions(argc, argv,
									help, input_file, output_file, data_file, out_format))
	{
		usage();
		return 1;
	}
	
	if (help)
	{
		usage();
		return 0;
	}
		
  /* ACTION */

	Trimesh* mesh = new Trimesh();
	sldTrimesh sldFile(mesh);
	sldFile.setDoubletts(true);
	
	if (!sldFile.read(input_file))
	{
		cerr << "Could not read input from the file " << input_file << " !" << endl;
		delete mesh;
		return 2;
	}

  //writing the mesh
	if (output_file!="")
		if (!writeMesh(mesh,output_file,out_format))
		{
			cerr << "Error writing the output to " << output_file << endl;
			delete mesh;
			return 3;
		}

	//writing the data
	if (data_file!="")
	{
		ofstream datastream(data_file.c_str());
		if (!datastream)
		{
			cerr << "Could not write vertex data to " << data_file << endl;
			delete mesh;
			return 4;
		}

		Rectmat_<double>& matrix = sldFile.data();
		int i=0;

		//write column heads
		for(i=0;i<matrix.cno();++i)
			datastream << sldFile.getColName(i) << "\t";
		
		datastream << endl;
		
		for(i=0;i<matrix.rno();++i)
		{
			for(int j=0;j<matrix.cno();++j)
				datastream << matrix(i,j) << "\t";
			datastream << endl;
		}
		
		datastream.close();
	}
  
  delete mesh;
  return 0;
}
