/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#include <trimesh.hh>
#include <fileformats.hh>
#include <unistd.h>

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <cstdio>

#ifdef HAVE_STD
#  include <iostream>
#  include <fstream>
#else
#  include <iostream.h>
#  include <fstream.h>
#endif


//******************************************************************************
void usage(ostream& out)
//******************************************************************************
{
  out << "Trimesh library - surface information tool v 1.0 (c)NFI vienna 2002 \n" 
      << endl
      << "usage : surfinfo [-i <input> -f <format> -d] information\n"
      << endl
      << "The input is read from standard input or the file <input> if specified,\n"
      << "output is written to standard output or to the file <output>.\n" << endl; 
 
  out << "Options:\n"
      << "  -i <input>   read the data from the file <input>\n"
			<< "  -f <format>  input file format"
			<< "  -d           remove doubletts in the input file (could cause large runtimes)\n"
			<< "  -h           displays this screen\n"
			<< endl;

  out << "Possible pieces of information are one ore more of the following: "
			<< "  points       the number of points on the surface\n"
			<< "  triangles    the number of triangles on the surface\n"
			<< "  edges        the number of edges on the surface\n"
			<< "  area         the surface area\n"
			<< endl;
 
}

//******************************************************************************
int main(int argc, char* argv[])
	//******************************************************************************
{
  extern char *optarg;
  extern int optind;
  
  string file;   //filename of the input file
  trimesh_format in_format = own_format;   //input format

  bool error_flag = false;

  //option flags
  bool removedoubletts = false;
  bool verbose         = false;
  bool help            = false;

  int c;
  while ((c = getopt(argc, argv, "i:f:dh")) != -1)
  {
    char buffer[200];
    switch(c)
    {
      case 'i' :
      {
				if (!sscanf(optarg, "%s", buffer))
				{
					error_flag = true;
					break;
				}
	
				file = buffer;
				break;
      }

      case 'f' :
      {
				if (!sscanf(optarg, "%s", buffer))
				{
					error_flag = true;
					break;
				}
	
				for(int i=0;i<N_FILEFORMATS;++i)
					if (string(buffer)==trimesh_formatnames[i]) 
					{
						in_format = (trimesh_format) i;
						break;
					}
      }
      
      case 'd':
      {
				removedoubletts = true;
				break;
      }

      case 'h':
      {
				help = true;
				break;
      }

      case '?':
				error_flag = true;
    }
  }
  if (error_flag) 
  {
    usage(cerr);
    return 1;
  }

  if (help)
  {
    usage(cout);
    return 0;
  }

  Trimesh* mesh;
  
  //reading the mesh

	mesh = readMesh(file,in_format,removedoubletts);
	
  if (!mesh)
  {
		if (file=="") file="stdin"; 
    cerr << "Error reading the input from " << file << "!"<< endl;
    return 2;
  }

	while (optind<argc)
	{
    string information = argv[optind++];

		if (information == "points")
			cout << mesh->nVertices() << endl;
		else if (information == "triangles")
			cout << mesh->nFaces() << endl;
		else if (information == "edges")
			cout << mesh->nEdges() << endl;
		else if (information == "area")
			cout << mesh->surfaceArea() << endl;
		else
		{
			usage(cerr);
			return 3;
		}
	}
  
  delete mesh;
  return 0;
}
