m4_include(RazorBack.m4)
m4_include(boost.m4)
m4_include(doctools.m4)

AC_DEFUN([AC_CXX_NAMESPACES],
[AC_CACHE_CHECK(whether the compiler implements namespaces,
ac_cv_cxx_namespaces,
[AC_LANG_SAVE
 AC_LANG_CPLUSPLUS
 AC_TRY_COMPILE([namespace Outer { namespace Inner { int i = 0; }}],
                [using namespace Outer::Inner; return i;],
 ac_cv_cxx_namespaces=yes, ac_cv_cxx_namespaces=no)
 AC_LANG_RESTORE
])
if test "$ac_cv_cxx_namespaces" = yes; then
  AC_DEFINE(HAVE_NAMESPACES,,[define if the compiler implements namespaces])
fi
])

AC_DEFUN([AC_CXX_HAVE_STD],
[AC_CACHE_CHECK(whether the compiler supports ISO C++ standard library,
ac_cv_cxx_have_std,
[AC_REQUIRE([AC_CXX_NAMESPACES])
 AC_LANG_SAVE
 AC_LANG_CPLUSPLUS
 AC_TRY_COMPILE([#include <iostream>
#include <map>
#include <iomanip>
#ifdef HAVE_NAMESPACES
using namespace std;
#endif],[return 0;],
 ac_cv_cxx_have_std=yes, ac_cv_cxx_have_std=no)
 AC_LANG_RESTORE
])
if test "$ac_cv_cxx_have_std" = yes; then
  AC_DEFINE(HAVE_STD,,[define if the compiler supports ISO C++ standard library])
fi
])

AC_DEFUN([AC_CXX_HAVE_STL],
[AC_CACHE_CHECK(whether the compiler supports Standard Template Library,
ac_cv_cxx_have_stl,
[AC_REQUIRE([AC_CXX_NAMESPACES])
 AC_LANG_SAVE
 AC_LANG_CPLUSPLUS
 AC_TRY_COMPILE([#include <list>
#include <deque>
#ifdef HAVE_NAMESPACES
using namespace std;
#endif],[list<int> x; x.push_back(5);
list<int>::iterator iter = x.begin(); if (iter != x.end()) ++iter; return 0;],
 ac_cv_cxx_have_stl=yes, ac_cv_cxx_have_stl=no)
 AC_LANG_RESTORE
])
if test "$ac_cv_cxx_have_stl" = yes; then
  AC_DEFINE(HAVE_STL,,[define if the compiler supports Standard Template Library])
  CMDSTD=-D_STANDARD_C_PLUS_PLUS
fi
])

AC_DEFUN([LIBTOOLIZE_VERSION],
[
  #retrieve the major minor and version step
  MAJOR=`expr match "$PACKAGE_VERSION" ['\([0-9]*\)\.[0-9]*\.[0-9]*']`			
	MINOR=`expr match "$PACKAGE_VERSION" ['[0-9]*\.\([0-9]*\)\.[0-9]*']`			
	VSTEP=`expr match "$PACKAGE_VERSION" ['[0-9]*\.[0-9]*\.\([0-9]*\)']`			
 
	#calculate the libtoolize version system
	INTERFACE=`expr $MINOR + $MAJOR`
	echo $INTERFACE
	AGE=`expr $INTERFACE - $MAJOR`
	echo $AGE
	IMPLEMENTATION=$VSTEP	

	AC_SUBST(MAJOR)
	AC_SUBST(MINOR)
	AC_SUBST(VSTEP)

	AC_SUBST(INTERFACE)
	AC_SUBST(IMPLEMENTATION)
	AC_SUBST(AGE)

	AC_MSG_NOTICE("the libtool version-info string is: $INTERFACE:$IMPLEMENTATION:$AGE")
])

AC_DEFUN([AC_LIBRARY_VERSION],
[
	LIBRARY_VERSION=$1

  #retrieve the major minor and version step
  INTERFACE=`expr match "$LIBRARY_VERSION" ['\([0-9]*\):[0-9]*:[0-9]*']`			
	IMPLEMENTATION=`expr match "$LIBRARY_VERSION" ['[0-9]*:\([0-9]*\):[0-9]*']`			
	AGE=`expr match "$LIBRARY_VERSION" ['[0-9]*:[0-9]*:\([0-9]*\)']`			
 
	#calculate the libtoolize version system
	MAJOR=`expr $INTERFACE - $AGE`
	MINOR=`expr $INTERFACE - $MAJOR`
	VSTEP=$IMPLEMENTATION

	AC_SUBST(MAJOR)
	AC_SUBST(MINOR)
	AC_SUBST(VSTEP)
	AC_SUBST(LIBRARY_VERSION)

	AC_MSG_NOTICE("the library is: $MAJOR.$MINOR.$VSTEP")
])
