/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#ifndef CONNTABTEMPL_CC
#define CONNTABTEMPL_CC


//*******************************************************************************
template<class Out>
int conntab::walkonBorder(Out ins, cVertex* startpoint)
//*******************************************************************************
{
  Vertex act_v;  //actual vertex on the path
  int c =0;

  if (startpoint)
  {
    //look for cvertex in the cvertexmap
    cvertexmap_it vertex = cvertexmap.find(*startpoint);

    //check if the cvertex is available
    if (vertex==cvertexmap.end()) throw unknown_cvertex(*startpoint);
  }
  else
  {
    pair<vertex_it, vertex_it> vits = vertices(ctab_graph);
    //find the first vertex on any border
    vertex_it border_v = find_if(vits.first,vits.second,
				 check_vertex_border(*this));

    //if there is no border then return with 0 vertices added to the list
    if (border_v==vits.second)
      return 0;
    
    act_v = *border_v;
  }

  Vertex first_v = act_v;

  //walk along the border
  do
  {
    //insert the vertex in the walk list
    *ins++ = vertexvert_map[act_v];
    
    //increase the count of vertices added to the list
    ++c;

    pair<outedges_it, outedges_it> outits = out_edges(act_v, ctab_graph);
    //find the edge leading to another vertex on the border
    outedges_it walk_edge = find_if(outits.first, outits.second,
				    check_edge_border(*this));
				   
    //if no further edge could be found the structure is inconsistent
    if (walk_edge==outits.second)
      throw inconsistent_structure();

    //move to next vertex
    act_v = target(*walk_edge, ctab_graph);
				    
  } while (act_v!=first_v);
  
  return c;
}

//*******************************************************************************
template<class Out>
int conntab::getBorderFaces(Out ins)
//*******************************************************************************
{
  set<cVertex> borderfaces;
  int c = 0;

  pair<edge_it, edge_it> edge_its = edges(ctab_graph);
  for(edge_it e=edge_its.first;e!=edge_its.second;++e)
    if (edgeborder_map[*e])
    {
      borderfaces.insert(edgeface_map[*e]);
      borderfaces.insert(edgelost_map[*e]);
      c += 2;
    }

  copy(borderfaces.begin(),borderfaces.end(),ins);

  return c;
}


//*******************************************************************************
template<class Out>
int conntab::getBorderVertices(Out ins)
//*******************************************************************************
{
  int c = 0;
  
  pair<vertex_it, vertex_it> vits = vertices(ctab_graph);
  for(vertex_it v=vits.first;v!=vits.second;++v)
    if (vertexborder_map[*v])
    {
      *ins++ = vertexvert_map[*v];
      ++c;
    }
  
  return c;
}

#endif
