/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#include "face.hh"

//*******************************************************************************
void Face::calcNormal()
//*******************************************************************************
{
  Vector_<double> a(3), b(3);
  normal.dim(3);
 
	a = *points[1].second - *points[0].second;
	b = *points.back().second - *points[0].second;

  normal[0] = a[1]*b[2]-a[2]*b[1];
  normal[1] = b[0]*a[2]-a[0]*b[2];
  normal[2] = a[0]*b[1]-a[1]*b[0];
}

//*******************************************************************************
int Face::addPoint(const Vertex ix, pointpos_t* pointpos)
//*******************************************************************************
{
  if ((pointpos->dim()!=3) || (ix<0))
    return -1;
  else
  {
    point_t newpoint(ix,pointpos);
    points.push_back(newpoint);

    //with 3 points the surface normal can be calculated
    //(every other point must lie in the same plain by definition
    if (points.size()==3) calcNormal();
    
    return points.size();
  }
}

//*******************************************************************************
int Face::insertPoint(const Vertex& before, 
		      const Vertex& ix, pointpos_t* newpointpos)
//*******************************************************************************
{ 
  point_t newpoint(ix,newpointpos);
  
  for(vector<point_t>::iterator p=points.begin();p!=points.end();++p)
    if (p->first==before)
    {
      points.insert(p,newpoint);
      return points.size()-2;
    }
  return -1;
}


//*******************************************************************************
Face::Vertex Face::getSuccessor(const Vertex& point) const
//*******************************************************************************
{
  for(vector<point_t>::const_iterator p=points.begin();p!=points.end();++p)
    if (p->first==point)
    {
      p++;
      if (p==points.end()) p = points.begin();
      return p->first;
    }
  
  return 0;
}

//*******************************************************************************
Face::Vertex Face::getPredecessor(const Vertex& point) const
//*******************************************************************************
{
  for(vector<point_t>::const_reverse_iterator p=points.rbegin();p!=points.rend();++p)
    if (p->first==point)
    {
      p++;
      if (p==points.rend()) p = points.rbegin();
      return p->first;
    }
  
  return 0;
}
