/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#ifndef FACE_HH
#define FACE_HH

#include <math.h>
#include <float.h>
#include <boost/graph/adjacency_list.hpp>
#include <Vector.hh>

#ifdef HAVE_CONFIG_H
#  include <config.h>
#  ifdef HAVE_STD
#    include <algorithm>
#  else
#    include <algo.h>
#  endif //HAVE_STD
#else //HAVE_CONFIG_H
#  include <algorithm>
#endif //HAVE_CONFIG_H

#ifdef HAVE_NAMESPACES
using namespace boost;
using namespace std;
using namespace RazorBack;
#endif //HAVE_NAMESPACES

typedef Vector_<double> pointpos_t;

class Face
{
private:
  typedef adjacency_list_traits<vecS,listS,directedS>::vertex_descriptor Vertex;
  typedef pair<Vertex,pointpos_t*> point_t;
  
  vector<point_t> points;
  Vector_<double> normal;
 
  void calcNormal();

  void clearPoints() 
  { 
    points.clear(); 
    normal.set_values(0); 
  }
  
public:

  Face():points(), normal() { };

  int nPoints() { return points.size(); }
  
  int addPoint(const Vertex ix, pointpos_t* newpoint);
  int insertPoint(const Vertex& before, const Vertex& ix, pointpos_t* newpointpos);

  Vector_<double> getNormal() const { return normal; }
  
  Vertex getPointIx(const int ix) const
  {
    if ((ix>=0) && (ix<points.size()))
      return points[ix].first;
    else
      return 0;
  }

  pointpos_t* getPointPos(const int ix) const
  {
    if ((ix>=0) && (ix<points.size()))
      return points[ix].second;
    else
      return 0;
  }

  Vertex getSuccessor(const Vertex& point) const;
  Vertex getPredecessor(const Vertex& point) const;
  

};

#endif
