/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#ifndef FACETABLE_CC
#define FACETABLE_CC

#include <math.h>

//*******************************************************************************
template<class triangle_t, class point_t>
faceTable<triangle_t,point_t>::
faceTable(const int triangles, 
					const int T_factor, const double width, const double height)
//*******************************************************************************
: face_table(),
	n_triangles(triangles), T(T_factor)
{
	n_cells = (int)ceil(sqrt((double) n_triangles)/T);
	step_x = width/n_cells;
	step_y = height/n_cells;
}

//*******************************************************************************
template<class triangle_t, class point_t>
faceTable<triangle_t,point_t>::~faceTable()
//*******************************************************************************
{
}

//*******************************************************************************
template<class triangle_t, class point_t>
faceTable<triangle_t, point_t>& 
faceTable<triangle_t,point_t>::operator+=(const triangle_t* triangle)
//*******************************************************************************
{
	//initialize the min/max variables with impossible values
	int min_x = n_cells+1;
	int min_y = n_cells+1;
	int max_x = -1;
	int max_y = -1;

	for(int i=0;i<3;++i)
	{
		point_t point = (*triangle)[i];
		
		//calculate the box of the vertex
		int ix_x = (int)(floor((1+point[0])/step_x));
		int ix_y = (int)(floor((1+point[1])/step_y));

		//check if the new index lies outside the box
		if (ix_x<min_x) min_x = ix_x;
		if (ix_y<min_y) min_y = ix_y;
		if (ix_x>max_x) max_x = ix_x;
		if (ix_y>max_y) max_y = ix_y;
	}

	//put the triangle in the boxes of the range (minx,miny | maxx, maxy)
	for(int i=min_y;i<=max_y;++i)
		for(int j=min_x;j<=max_x;++j)
		{
		  int key = j + n_cells*i;
			face_table.insert(pair<int, const triangle_t*>(key,triangle));
		}

	return *this;
}

//*******************************************************************************
template<class triangle_t, class point_t>
faceTable<triangle_t,point_t>::map_cits 
faceTable<triangle_t,point_t>::operator()(const double x, const double y) const
//*******************************************************************************
{
	map_cits result;
	
	if (((x>=-1) && (x<=step_x*n_cells)) && ((y>=-1) && (y<=step_y*n_cells)))
	{
		int ix_x = (int)floor((x+1)/step_x);
		int ix_y = (int)floor((y+1)/step_y);

		int key = ix_x + ix_y*n_cells;
		
		result = face_table.equal_range(key);
	}
	else
    result = face_table.equal_range(-1);

	return result;
}

#endif //FACETABLE_CC
