/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#include "fileformats.hh"

#ifdef HAVE_CONFIG_H
#  include <config.h>
#  ifdef HAVE_STD
#    include <fstream>
#  else
#    include <fstream.h>
#  endif //HAVE_STD
#else //HAVE_CONFIG_H
#  include <fstream>
#endif //HAVE_CONFIG_H

#include "trimesh.hh"
#include "trimeshio.hh"
#include "owntrimesh.hh"
#include "offtrimesh.hh"
#include "msmstrimesh.hh"
#include "geomtrimesh.hh"
#include "meshtrimesh.hh"
#include "bintrimesh.hh"
#include "sldtrimesh.hh"
#include "povtrimesh.hh"

//******************************************************************************
Trimesh* readMesh(const string& filename, const trimesh_format fmt, 
		  const bool doubletts)
//******************************************************************************
{
	bool result;

  Trimesh* mesh = new Trimesh();
  
  trimeshIO* ioclass;
  
  switch(fmt)
  {
    case own_format:
    {
      ioclass = new ownTrimesh(mesh);
      break;
    }
    
    case off_format:
    {
      ioclass = new offTrimesh(mesh);
      break;
    }

    case msms_format:
    {
      ioclass = new msmsTrimesh(mesh);
      break;
    }

    case geom_format:
    {
      //ioclass = new geomTrimesh(mesh);
			cerr << "Cannot read a geom format !" << endl;
			return (Trimesh*) 0;
      break;
    }

    case mesh_format:
    {
      //ioclass = new geomTrimesh(mesh);
			cerr << "Cannot read a mesh format !" << endl;
			return (Trimesh*) 0;
      break;
    }

    case bin_format:
    {
      ioclass = new binTrimesh(mesh);
      break;
    }

    case sld_format:
    {
      ioclass = new sldTrimesh(mesh);
      break;
    }

    case pov_format:
    {
      //ioclass = new povTrimesh(mesh);
			cerr << "Cannot read a pov format !" << endl;
			return (Trimesh*) 0;
      break;
    }
  }

  //read the object with doubletts or without
  ioclass->setDoubletts(doubletts);
	if (filename!="")
	{
		ifstream stream(filename.c_str());
		if (!stream) 
			result = false;
		else 	
		{
			result = ioclass->read(stream);
			stream.close();
		}
	}
	else
		result = ioclass->read(cin);

  if (result)
  {
    delete ioclass;
    return mesh;
  }
  else
  {
    delete ioclass;
    if (mesh) delete mesh;
    return (Trimesh*)0;
  }
    
}

//******************************************************************************
Trimesh* readMesh(istream& stream, const trimesh_format fmt, 
		  const bool doubletts)
//******************************************************************************
{
	bool result;

  Trimesh* mesh = new Trimesh();
  
  trimeshIO* ioclass;
  
  switch(fmt)
  {
    case own_format:
    {
      ioclass = new ownTrimesh(mesh);
      break;
    }
    
    case off_format:
    {
      ioclass = new offTrimesh(mesh);
      break;
    }

    case msms_format:
    {
      ioclass = new msmsTrimesh(mesh);
      break;
    }

    case geom_format:
    {
      //ioclass = new geomTrimesh(mesh);
			cerr << "Cannot read a geom format !" << endl;
			return (Trimesh*) 0;
      break;
    }

    case mesh_format:
    {
      //ioclass = new geomTrimesh(mesh);
			cerr << "Cannot read a mesh format !" << endl;
			return (Trimesh*) 0;
      break;
    }

    case bin_format:
    {
      ioclass = new binTrimesh(mesh);
      break;
    }

    case sld_format:
    {
      ioclass = new sldTrimesh(mesh);
      break;
    }

    case pov_format:
    {
      //ioclass = new povTrimesh(mesh);
			cerr << "Cannot read a pov format !" << endl;
			return (Trimesh*) 0;
      break;
    }

  }

	ioclass->setDoubletts(doubletts);
	result = ioclass->read(stream);

  if (result)
  {
    delete ioclass;
    return mesh;
  }
  else
  {
    delete ioclass;
    if (mesh) delete mesh;
    return (Trimesh*)0;
  }
    
}

//******************************************************************************
bool writeMesh(Trimesh* mesh, const string& filename, const trimesh_format fmt)
//******************************************************************************
{
  
  trimeshIO* ioclass;
	bool result;
  
  switch(fmt)
  {
    case own_format :
    {
      ioclass = new ownTrimesh(mesh);
      break;
    }
    
    case off_format :
    {
      ioclass = new offTrimesh(mesh);
      break;
    }

    case msms_format :
    {
      ioclass = new msmsTrimesh(mesh);
      break;
    }

    case geom_format :
    {
      ioclass = new geomTrimesh(mesh);
      break;
    }

    case mesh_format :
    {
      ioclass = new meshTrimesh(mesh);
      break;
    }

    case bin_format:
    {
      ioclass = new binTrimesh(mesh);
      break;
    }

    case sld_format:
    {
      ioclass = new sldTrimesh(mesh);
      break;
    }

    case pov_format:
    {
      ioclass = new povTrimesh(mesh);
      break;
    }

  }

  //write the object to the stream
	if (filename!="")
	{
		ofstream stream(filename.c_str());
		if (!stream) 
			result = false;
		else 	
		{
			result = ioclass->write(stream);
			stream.close();
		}
	}
	else
		result = ioclass->write(cout);

  delete ioclass;
  return result;
    
}

//******************************************************************************
bool writeMesh(Trimesh* mesh, ostream& stream, const trimesh_format fmt)
//******************************************************************************
{
  
  trimeshIO* ioclass;
	bool result;
  
  switch(fmt)
  {
    case own_format :
    {
      ioclass = new ownTrimesh(mesh);
      break;
    }
    
    case off_format :
    {
      ioclass = new offTrimesh(mesh);
      break;
    }

    case msms_format :
    {
      ioclass = new msmsTrimesh(mesh);
      break;
    }

    case geom_format :
    {
      ioclass = new geomTrimesh(mesh);
      break;
    }

    case mesh_format :
    {
      ioclass = new meshTrimesh(mesh);
      break;
    }

    case bin_format:
    {
      ioclass = new binTrimesh(mesh);
      break;
    }

    case sld_format:
    {
      ioclass = new sldTrimesh(mesh);
      break;
    }

    case pov_format:
    {
      ioclass = new povTrimesh(mesh);
      break;
    }

  }

  //write the object to the stream
	result = ioclass->write(stream);

  delete ioclass;
  return result;
    
}
