/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

/**
 * \file fileformats.hh
 * The fileformats.hh header includes all available file format header files.
 * If you want to include all the file formats, available for the Trimesh library
 * just include this header file. If you have programed new file formats, feal 
 * free to include them in this header as well
 */

#ifndef FILEFORMATS_HH
#define FILEFORMATS_HH

#ifdef HAVE_CONFIG_H
#  include <config.h>
#  ifdef HAVE_STD
#    include <iostream>
#    include <string>
#  else
#    include <iostream.h>
#  endif //HAVE_STD
#else //HAVE_CONFIG_H
#  include <iostream>
#  include <string>
#endif //HAVE_CONFIG_H

class Trimesh;

const int N_FILEFORMATS = 8;

/**
 * \brief the fileformats enumeration is a specifier of all posiible file formats
 * 
 * If you add a new file format, please add a fileformat specifier for your
 * format to this type
 * 
 */
enum trimesh_format 
{ own_format = 0, off_format, msms_format, geom_format, mesh_format, bin_format, 
	sld_format, pov_format };

static const char* trimesh_formatnames[] =
{
  "own",
  "off",
  "msms",
  "geom",
  "mesh",
	"bin",
	"sld",
	"inc"
};

static const char* trimesh_formatdescriptions[] =
{
  "Trimesh Library own text files",
  "Geomview OFF files",
  "Michael Sanners MS files",
  "Centerstage description files ",
  "MEdit 3D object files",
	"Trimesh Library binary files",
	"MOLCAD surface description files",
	"PovRay surface definition"

};

Trimesh* readMesh(const std::string& filename, const trimesh_format fmt = own_format,
									const bool doubletts=false);
Trimesh* readMesh(std::istream& stream, const trimesh_format fmt = own_format,
									const bool doubletts=false);
bool writeMesh(Trimesh* mesh, std::ostream& stream, const trimesh_format fmt = own_format);
bool writeMesh(Trimesh* mesh, const std::string& filename, const trimesh_format fmt = own_format);

#endif
