/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#include <geomtrimesh.hh>
#include <float.h>

#ifdef HAVE_CONFIG_H
#  include <config.h>
#  ifdef HAVE_STD
#    include <fstream>
#    include <sstream>
#    include <cstdio>
#    include <iostream>
#    include <iomanip>
#    include <algorithm>
#  else
#    include <fstream.h>
#    include <sstream.h>
#    include <cstdio.h>
#    include <iostream.h>
#    include <iomanip.h>
#    include <algo.h>
#    include <map.h>
#  endif //HAVE_STD
#else //HAVE_CONFIG_H
#  include <fstream>
#  include <sstream>
#  include <cstdio>
#  include <iostream>
#  include <iomanip>
#  include <algorithm>
#endif //HAVE_CONFIG_H

#include <Distrapprox.hh>

//*******************************************************************************
string geomTrimesh::makeColorEntry(const int v) const
//*******************************************************************************
{
	string colorstr = "";

	if (fColors.size()==0) return colorstr;

	if (fDimParameter)
	{
		//with dimming
		char colorchar[46] = "";		
		sprintf(colorchar, "{1-(1-%.3f)*df 1-(1-%.3f)*df 1-(1-%.3f)*df}",	
						fColors[v].r, fColors[v].g, fColors[v].b);
		colorstr = colorchar;
	}
	else
	{
		//without dimming
		char colorchar[19] = "";
		sprintf(colorchar, "{%.3f %.3f %.3f}",	
						fColors[v].r, fColors[v].g, fColors[v].b);
		colorstr = colorchar;
	}

	return colorstr;
}

//*******************************************************************************
void geomTrimesh::calcRangeColor(double value, color_t& color)
//*******************************************************************************
{
	//clamp the value in the range [0,1]
	if (value<0) value == 0;
	if (value>1) value == 1;
	
	//the following code is taken from the Plot3D component in the SDL component
	//suite written by H. Lohninger in Delphi/Pascal.

  double redLow;
	double redHigh;
  double greenLow;
  double greenHigh;
	double blueLow;
  double blueHigh;

	//check if a three color scale is necassary 
	bool threecolors = (fMedColor.r>DBL_EPSILON) || 
		                 (fMedColor.g>DBL_EPSILON) || 
		                 (fMedColor.b>DBL_EPSILON);
	if (threecolors)
		if (value<0.5)
		{
			redLow    = fLowColor.r;
      redHigh   = fMedColor.r;
			greenLow  = fLowColor.g;
      greenHigh = fMedColor.g;
			blueLow   = fLowColor.b;
			blueHigh  = fMedColor.b;
		}
		else
		{
			value = value - 0.5;
			redLow    = fMedColor.r;
      redHigh   = fHighColor.r;
			greenLow  = fMedColor.g;
      greenHigh = fHighColor.g;
			blueLow   = fMedColor.b;
			blueHigh  = fHighColor.b;
		}
	else
	{
		value = value/2;
			redLow    = fLowColor.r;
      redHigh   = fHighColor.r;
			greenLow  = fLowColor.g;
      greenHigh = fHighColor.g;
			blueLow   = fLowColor.b;
			blueHigh  = fHighColor.b;
	}

  color.r = redLow + 2*value * (redHigh - redLow);
  color.g = greenLow + 2*value * (greenHigh - greenLow);
  color.b = blueLow + 2*value * (blueHigh - blueLow);
}


//*******************************************************************************
void geomTrimesh::getRangeColor(const int color, double& r, double& g, double& b)
//*******************************************************************************
{
	switch(color)
	{
		case 0 : 
		{
			r = fLowColor.r;
			g = fLowColor.g;
			b = fLowColor.b;
			break;
		}
			
		case 1 : 
		{
			r = fMedColor.r;
			g = fMedColor.g;
			b = fMedColor.b;
			break;
		}

		case 2 : 
		{
			r = fHighColor.r;
			g = fHighColor.g;
			b = fHighColor.b;
			break;
		}
	}
}

//*******************************************************************************
void geomTrimesh::setRangeColor(const int color, 
																const double r, const double g, const double b)
//*******************************************************************************
{
	switch(color)
	{
		case 0 : fLowColor(r,g,b); break;
		case 1 : fMedColor(r,g,b); break;
		case 2 : fHighColor(r,g,b);	break;
	}
}	


//******************************************************************************
bool geomTrimesh::read(const string& aFile)
//******************************************************************************
{
	return false;
//   //open the File
//   ifstream stream(aFile.c_str());
//   if (!stream)
//   {
//     cerr << "Coordinate file: " << aFile << " not found" << endl;
//     return false;
//   }

//   bool result = read(stream);  

//   stream.close();
//   return result;
}


//******************************************************************************
bool geomTrimesh::read(istream& aStream)
//******************************************************************************
{
//   mesh_t& Mesh = getMesh();
//   vector<Triangle>& Triangles = getTriangles();
//   map< int, vector<int> >& vertfaceIndex = getvertfaceIndex();
  
//   char line_buffer[200];

//   Vector_<double> xyz(3);  // Coordinates
//   Vector_<double> nxyz(3); // Normals

// 	map<int, Vector_<double>> vertex_map;
// 	list<vector<int> > int_map;

	return true;
}

//******************************************************************************
bool geomTrimesh::write(const string& aFile)
//******************************************************************************
{
  //open the File
  ofstream stream(aFile.c_str());
  if (!stream)
  {
    cerr << "File: " << aFile << " could not be created" << endl;
    return false;
  }

  bool result = write(stream);  

  stream.close();
  return result;
}


//******************************************************************************
bool geomTrimesh::write(ostream& aStream)
//******************************************************************************
{
  mesh_t& Mesh = getMesh();
  vector<Triangle>& Triangles = getTriangles();
  map< int, vector<int> >& vertfaceIndex = getvertfaceIndex();

  char headline[200];

  //write format descriptor
  
  //write the vertices
  aStream << "Vertices {" << endl;

  for(Trimesh::vertex_it v=mesh_structure->begin();v!=mesh_structure->end();++v)
  { 
    char linebuffer[200];
    
    sprintf(linebuffer, "  %d: (%9.3f, %9.3f, %9.3f)\n",
						(*data_map)[*v].ownIndex, 
						(*data_map)[*v].coordinates[0], 
						(*data_map)[*v].coordinates[1], 
						(*data_map)[*v].coordinates[2]);
		
    aStream << linebuffer;
  }

	aStream << "}"  << endl << endl;

  //write the triangles
  aStream << "Faces {" << endl;

  for(Trimesh::FaceIter f=mesh_structure->beginFace();f!=mesh_structure->endFace();++f)
  {
    char linebuffer[200];

		if (fVertexColoring)
			sprintf(linebuffer, "  {%6d %6d %6d} <- { vcolors {%s %s %s}}\n",
							f->getVertexIndex(0),
							f->getVertexIndex(1),
							f->getVertexIndex(2),
							makeColorEntry(f->getVertexIndex(0)).c_str(),
							makeColorEntry(f->getVertexIndex(1)).c_str(),
							makeColorEntry(f->getVertexIndex(2)).c_str());
		else
			sprintf(linebuffer, "  {%6d %6d %6d}\n",
							f->getVertexIndex(0),
							f->getVertexIndex(1),
							f->getVertexIndex(2));
			

    aStream << linebuffer;
  }
	aStream << "}"  << endl;

  return true;
}

//*******************************************************************************
int geomTrimesh::setColorColumn(const int col)
//*******************************************************************************
{
	int oldValue = fColorColumn;
	fColorColumn = col;
	return oldValue;
}

//*******************************************************************************
bool geomTrimesh::enableColoring(const bool coloring)
//*******************************************************************************
{
	bool oldValue = fVertexColoring;
	fVertexColoring = coloring;
	return oldValue;
}

//*******************************************************************************
bool geomTrimesh::enableDimParameter(const bool dimming)
//*******************************************************************************
{
	bool oldValue = fDimParameter;
	fDimParameter = dimming;
	return oldValue;	
}

//*******************************************************************************
bool geomTrimesh::enableOutline(const bool outline)
//*******************************************************************************
{
	bool oldValue = fOutline;
	fOutline = outline;
	return oldValue;	
}

//*******************************************************************************
void geomTrimesh::calcColors(double max, double min)
//*******************************************************************************
{
	Vector_<double> col = fData.col(fColorColumn);
	//resize the color vector
	fColors.resize(mesh_structure->nVertices());

	colors_it c = fColors.begin();
	for(Trimesh::vertex_it v=mesh_structure->begin();v!=mesh_structure->end();
			++v, ++c)
	{
		int ix = mesh_structure->vertMasterIx(*v);
		double value = (col[ix]-min)/(max-min);

		calcRangeColor(value,*c);
	}

}

//*******************************************************************************
void geomTrimesh::calcColors(RazorBack::Distrapprox_* dist)
//*******************************************************************************
{
	Vector_<double> col = fData.col(fColorColumn);

	dist->evaluate();

	//resize the color vector
	fColors.resize(mesh_structure->nVertices());
	colors_it c = fColors.begin();

	for(Trimesh::vertex_it v=mesh_structure->begin();v!=mesh_structure->end();
			++v, ++c)
	{
		//extract a the cdf value for i
		int ix = mesh_structure->vertMasterIx(*v);
		double cdf = dist->cdf(col[ix]);
		calcRangeColor(cdf, *c);
	}
}
