/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#ifndef MESHTRIMESH_HH
#define MESHRIMESH_HH

#include "trimeshio.hh"

/**
 * \brief trimesh class, able to read and write triangulated mesh objects in
 * the library own format.
 *
 * The file format used by this library itself is described on the file Format pages
 * under \ref meshformat_page "MEdit Mesh File Format".
 */
class meshTrimesh : public trimeshIO
{
public:

  /**
   * \brief constructor initializing msmsTrimesh with a Trimesh object
   */
  meshTrimesh(Trimesh* aTrimesh): trimeshIO(aTrimesh) { }

  /** 
   * \brief there is a stream handling for library own format files.
   */
  bool handleStreams() const { return true; }


  /**
   * \brief reads a surface mesh object in library own format from a file 
   * specified in \p aFile
   *
   */
  bool read(const string& aFile);
  
  /**
   * \brief reads a surface mesh object in library own format from the stream
   * specified in \p aStream
   */
  bool read(istream& aStream);

  /**
   * \brief writes a surface mesh object in library own format into a file 
   * specified in \p aFile
   *
   */
  bool write(const string& aFile);

  /**
   * \brief writes a surface mesh object in library own format into a stream 
   * specified in \p aStream
   *
   */
  bool write(ostream& aStream);
};

#endif //MESHTRIMESH_HH


/**
 * \page meshformat_page MEdit Mesh File Format
 *
 * The file format, which is used by the Medit program.
 *
 * \section meshfile_header File Header:
 * The file starts with two descriptor lines:
 * \code
 *   MeshVersionFormatted 1 
 *   Dimension 3
 * \endcode
 * \section meshfile_meshdescription Mesh Description:
 * to be written
 */
