/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#include "normcorr_matching.hh"
#include "harmonic_image.hh"

#include <math.h>

#include <Rectmat.hh>
#include <Vector.hh>

#ifdef HAVE_CONFIG_H
#  include <config.h>
#  ifdef HAVE_STD
#    include <vector>
#    include <algorithm>
#  else
#    include <vector.h>
#    include <algo.h>
#  endif //HAVE_STD
#else //HAVE_CONFIG_H
#  include <vector>
#  include <algorithm>
#endif //HAVE_CONFIG_H

//*******************************************************************************
double normcorr_matching::operator()(const harmonicImage& key2)
//*******************************************************************************
{

	if (search_array==key1->search_end())	throw out_of_range("search_array");

  int res1 = key1->getResolution();
	int res2 = key2.getResolution();

	if ((res1==res2) && key2.image_data.cno()==search_array->cno()) 
	{
		int N = 0;

		//required sums with p and q

		double sum_q = 0;
		double sum_square_q = 0;
		double sum_pq = 0;
		double sum_p = 0;
		double sum_square_p = 0;

		double invalid1 = key1->getInvalidValue();
		double invalid2 = key2.getInvalidValue();

		//computing the sums for q q^2 and p*q
		
		for(int i=0;i<key2.image_data.rno();++i)
			if ((key2.image_data(i,0)>(invalid2+1)) && ((*search_array)(i,0)>(invalid1+1)))
				for(int j=0;j<key2.image_data.cno();++j)
					if ((cols==0) || ((cols >> j) & 1))
					{
						++N;

						double p= (*search_array)(i,j);
 						double q= key2.image_data(i,j);

						sum_q += q;
						sum_square_q += q * q;

						sum_p += p;
						sum_square_p += p * p;
					
						sum_pq += p * q;
					}
		
		double r = N*sum_pq - sum_p*sum_q;  //calculating first part of r
		r = r/sqrt(fabs(N*sum_square_p - sum_p*sum_p) * 
							 fabs(N*sum_square_q - sum_q*sum_q));

		correlations.push_back(r);

#ifdef LT_DEBUGGING		
		// cout << N << "\t" << r << endl;
#endif //LT_DEBUGGING
		
		//jump to the next search_array
		++search_array;
		return r;
		
	}
	else throw match_diffresolution();

	return -2;

}

