/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#ifndef NORMCORR_MATCHING_HH
#define NORMCORR_MATCHING_HH

#include "img_matching.hh"

/**
 * \brief base class for all harmonic image mapping functors
 *
 * They must obey the following concept:
 *
 * operator(key1, key2) compares two harmonic images
 * copy_corrfunction(OUT) fills the sequence OUT with the single correlation 
 * values
 */
class normcorr_matching: public img_matching
{
public:
	/**
	 * \brief standard constructor initializing the correlations list
	 */
	normcorr_matching(const harmonicImage* key = 0, const int cols = 0): img_matching(key,cols) { }

	/**
	 * \brief function operator comparing two harmonic images
	 *
	 * returns a correlation coefficient between zero and one
	 *
	 * \param key2 harmonic images to be compared
	 */
	double operator()(const harmonicImage& key2);
};

#endif //IMG_MATCHING_HH
