/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#ifndef PATCHGRAPH_HH
#define PATCHGRAPH_HH

#include <boost/config.hpp>
#include <boost/graph/adjacency_list.hpp>
#include <boost/graph/properties.hpp>
#include <boost/property_map.hpp>

#include "trimesh.hh"

#ifdef HAVE_NAMESPACES
namespace boost
{
#endif

  enum vertex_pvert_t     { vertex_pvert     = 600 };

  BOOST_INSTALL_PROPERTY(vertex,pvert);

#ifdef HAVE_NAMESPACES
};
#endif

class patchgraph_traits
{
public:
  /**
   * \brief a vertex type for the patch graph.
   */
  typedef adjacency_list_traits<listS,listS,directedS>::vertex_descriptor pvertex_t;

  /**
   * \brief an edge type for the patch graph.
   */
  typedef adjacency_list_traits<listS,listS,directedS>::edge_descriptor pedge_t;

  /**
   * \brief a vertex iterator for the patch graph.
   */
  typedef graph_traits<adjacency_list<listS,listS,directedS> >::vertex_iterator pvertex_it;

  /**
   * \brief an edge iterator for the patch graph.
   */
  typedef graph_traits<adjacency_list<listS,listS,directedS> >::edge_iterator pedge_it;

  /**
   * \brief specifies the conflictgraph Vertex of the vertex
   */
  typedef property<vertex_pvert_t, Trimesh::vertex_t> VertexVert;


  /**
   * \brief reciproce number of intersecting vertices of two adjacent patches
   *
   * The length of the edges is stored as the weights (colors) of the mesh's
   * edges.
   */
  typedef property<edge_weight_t, double> EdgeLength;

  /**
   * \brief typdefinition for the patch graph.
	 * 
	 * The patch graph consists of a set of undirected edges, a list of vertices and
	 * has a single vertex property (the index of the central vertex of the patch) and
	 * a single edge property ( the reciproce number of vertices which belong to two
	 * adjacent patches).
   */
  typedef adjacency_list<setS, listS, undirectedS, VertexVert, EdgeLength> patchgraph_t;

	/**
	 * \brief property map, containing the patchs' central vertices
	 */
	typedef property_map<patchgraph_t, vertex_pvert_t>::type cvertex_map_t;

  /**
   * \brief property map for the edge lengths (weights)
   */
  typedef property_map<patchgraph_t, edge_weight_t>::type edgelenght_map_t;

};


#endif //PATCHGRAPH_HH
