/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#ifndef TRIMESH_TEMPL_CC
#define TRIMESH_TEMPL_CC

//*******************************************************************************
template<class Out>
void Trimesh::commonTriangles(vertex_t v, vertex_t u, Out ins) const
//*******************************************************************************
{
	int vix = vertIndex(v,false);
	int uix = vertIndex(u,false);
	set<int> v_triangles(beginFaceIx(vix),endFaceIx(vix));
	set<int> u_triangles(beginFaceIx(uix),endFaceIx(uix));

	set_intersection(v_triangles.begin(),v_triangles.end(),
									 u_triangles.begin(),u_triangles.end(),
									 ins);
}

//******************************************************************************
template<class Out>
size_t Trimesh::onering(const vertex_t& v, Out ins) const
//******************************************************************************
{
	/* INITIALIZATION */
	size_t c = 0;

	//coordinates and normal of v
	Vector_<double> nv  = vertNormals(v);
	Vector_<double> pv = vertCoordinates(v); 

	//receive the adjacency of the desired vertex
	pair<adjacency_it, adjacency_it> v_aits = adjacent_vertices(v, Mesh);


	vertex_t w = *v_aits.first;
	vertex_t end = w;

	do 
	{
		//insert into output sequence
		// cout << w << " ";
		*ins++ = w;
		++c;
		w = nextVertex(v,w);

	} while (w != end);
	
	// cout << endl;
	return c;
}

#endif
