/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#include "trimeshio.hh"

//*******************************************************************************
void trimeshIO::reverseBytes(char* seq, const int len)
//*******************************************************************************
{
	int steps = floor((double)len/2);
	register char help;
	
	for(int i=0;i<steps;++i)
	{
		help = seq[i];
		seq[i] = seq[len-1-i];
		seq[len-1-i] = help;
	}
}

//*******************************************************************************
size_t trimeshIO::writeInt(ostream& stream, const int value, bool reverse)
//*******************************************************************************
{
	if (reverse) reverseBytes((char*) &value, sizeof(int));
	stream.write((char*) &value, sizeof(int));

	return sizeof(int);
}

//*******************************************************************************
size_t trimeshIO::writeShort(ostream& stream, const short value, bool reverse)
//*******************************************************************************
{
	if (reverse) reverseBytes((char*) &value, sizeof(short));
	stream.write((char*) &value, sizeof(short));	

	return sizeof(short);
}

//*******************************************************************************
size_t trimeshIO::writeDouble(ostream& stream, const double value, bool reverse)
//*******************************************************************************
{
	if (reverse) reverseBytes((char*) &value, sizeof(double));
	stream.write((char*) &value, sizeof(double));		

	return sizeof(double);
}

//*******************************************************************************
size_t trimeshIO::writeFloat(ostream& stream, const float value, bool reverse)
//*******************************************************************************
{
	if (reverse) reverseBytes((char*) &value, sizeof(float));
	stream.write((char*) &value, sizeof(float));	

	return sizeof(float);
}

//*******************************************************************************
size_t trimeshIO::writeIntArr(ostream& stream, const int* value, int len, 
															bool reverse)
//*******************************************************************************
{
	if (reverse)
		for(int i=0;i<len;++i) reverseBytes((char*) &value[i], sizeof(int));
	stream.write((char*) value, sizeof(int) * len);		

	return sizeof(int) * len;
}

//*******************************************************************************
size_t trimeshIO::writeShortArr(ostream& stream, const short* value, int len, 
																bool reverse)
//*******************************************************************************
{
	if (reverse)
		for(int i=0;i<len;++i) reverseBytes((char*) &value[i], sizeof(short));
	stream.write((char*) value, sizeof(short) * len);			

	return sizeof(short) * len;
}

//*******************************************************************************
size_t trimeshIO::writeDoubleArr(ostream& stream, const double* value, int len,
																 bool reverse)
//*******************************************************************************
{
	if (reverse)
		for(int i=0;i<len;++i) reverseBytes((char*) &value[i], sizeof(double));
	stream.write((char*) value, sizeof(double) * len);			

	return sizeof(double) * len;
}

//*******************************************************************************
size_t trimeshIO::writeFloatArr(ostream& stream, const float* value, int len,
																bool reverse)
//*******************************************************************************
{
	if (reverse)
		for(int i=0;i<len;++i) reverseBytes((char*) &value[i], sizeof(float));
	stream.write((char*) value, sizeof(float) * len);		

	return sizeof(float) * len;
}

//*******************************************************************************
size_t trimeshIO::readInt(istream& stream, int& value, bool reverse)
//*******************************************************************************
{
	stream.read((char*) &value, sizeof(int));	
	if (reverse) reverseBytes((char*) &value, sizeof(int));

	return sizeof(int);
}

//*******************************************************************************
size_t trimeshIO::readShort(istream& stream, short& value, bool reverse)
//*******************************************************************************
{
	stream.read((char*) &value, sizeof(short));	
	if (reverse) reverseBytes((char*) &value, sizeof(short));

	return sizeof(short);
}

//*******************************************************************************
size_t trimeshIO::readDouble(istream& stream, double& value, bool reverse)
//*******************************************************************************
{
	stream.read((char*) &value, sizeof(double));	
	if (reverse) reverseBytes((char*) &value, sizeof(double));

	return sizeof(double);
}

//*******************************************************************************
size_t trimeshIO::readFloat(istream& stream, float& value, bool reverse)
//*******************************************************************************
{
	stream.read((char*) &value, sizeof(float));	
	if (reverse) reverseBytes((char*) &value, sizeof(float));

	return sizeof(float);
}

//*******************************************************************************
size_t trimeshIO::readIntArr(istream& stream, int* value, int len, bool reverse)
//*******************************************************************************
{
	stream.read((char*) value, sizeof(int) * len);		
	if (reverse)
		for(int i=0;i<len;++i) reverseBytes((char*) &value[i], sizeof(int));

	return sizeof(int) * len;
}

//*******************************************************************************
size_t trimeshIO::readShortArr(istream& stream, short* value, int len,
															 bool reverse)
//*******************************************************************************
{
	stream.read((char*) value, sizeof(short) * len);		
	if (reverse)
		for(int i=0;i<len;++i) reverseBytes((char*) &value[i], sizeof(short));

	return sizeof(short) * len;
}

//*******************************************************************************
size_t trimeshIO::readDoubleArr(istream& stream, double* value, int len,
																bool reverse)
//*******************************************************************************
{
	stream.read((char*) value, sizeof(double) * len);		
	if (reverse)
		for(int i=0;i<len;++i) reverseBytes((char*) &value[i], sizeof(double));

	return sizeof(double) * len;
}

//*******************************************************************************
size_t trimeshIO::readFloatArr(istream& stream, float* value, int len, 
															 bool reverse)
//*******************************************************************************
{
	stream.read((char*) value, sizeof(float) * len);		
	if (reverse)
		for(int i=0;i<len;++i) reverseBytes((char*) &value[i], sizeof(float));

	return sizeof(float) * len;
}

//******************************************************************************
trimeshIO::trimeshIO()
//******************************************************************************
: mesh_structure(0), removeDoubletts(false), doublettix(), fData(0,0),
  data_map(0), edgelengths_map(0)
{}

//******************************************************************************
trimeshIO::trimeshIO(Trimesh* aTrimesh)
//******************************************************************************
: mesh_structure(aTrimesh), removeDoubletts(false),  doublettix(), fData(0,0),
  data_map(&mesh_structure->data_map),
  edgelengths_map(&mesh_structure->edgelengths_map)
{  
}

//******************************************************************************
bool trimeshIO::setDoubletts(const bool value)
//******************************************************************************
{
  bool oldvalue = removeDoubletts;
  removeDoubletts = value;
  
  return oldvalue;
}

//*******************************************************************************
string trimeshIO::getColName(const int ix) const
//*******************************************************************************
{
	if ((ix>=0) && (ix<fColumnHeads.size()))
		return fColumnHeads[ix];
	else
		return "";
}

//*******************************************************************************
string trimeshIO::setColName(const int ix, const string& colname)
//*******************************************************************************
{
	if ((ix>=0) && (ix<fColumnHeads.size()))
	{
		string old_value = fColumnHeads[ix];
		fColumnHeads[ix] = colname;
		return old_value;
	}
	else
		return "";
	
}
