/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#include "xmlerrorreporter.hh"
#include <xercesc/sax/SAXParseException.hpp>
#include <xercesc/dom/DOMError.hpp>
#include <xercesc/dom/DOMLocator.hpp>

#include "conversion.hh"

#ifdef HAVE_NAMESPACES
using namespace std;
XERCESNS_USE
#endif 

#include <iostream>

//******************************************************************************
void ErrorReporter::warning(const SAXParseException& toCatch)
//******************************************************************************
{
	warnings_.push_back(SyntaxError(toCatch.getLineNumber(),
											toCatch.getColumnNumber(),
											XMLStr::convert(toCatch.getMessage())));
}

//******************************************************************************
void ErrorReporter::error(const SAXParseException& toCatch)
//******************************************************************************
{
	errors_.push_back(SyntaxError(toCatch.getLineNumber(),
																toCatch.getColumnNumber(),
																XMLStr::convert(toCatch.getMessage())));
}

//******************************************************************************
void ErrorReporter::fatalError(const SAXParseException& toCatch)
//******************************************************************************
{
	error(toCatch);
}

//******************************************************************************
bool ErrorReporter::handleError(const DOMError& error)
//******************************************************************************
{
	switch (error.getSeverity())
	{
		case DOMError::DOM_SEVERITY_WARNING : 
		{
			cout << " DD warning:" <<  XMLStr::convert(error.getMessage()) << endl;
			warnings_.push_back(SyntaxError(error.getLocation()->getLineNumber(),
																			error.getLocation()->getColumnNumber(),
																			XMLStr::convert(error.getMessage())));
			return true;
		}
		case DOMError::DOM_SEVERITY_ERROR :
		case DOMError::DOM_SEVERITY_FATAL_ERROR :
		{
			cout << " DD error:" <<  XMLStr::convert(error.getMessage()) << endl;
			errors_.push_back(SyntaxError(error.getLocation()->getLineNumber(),
																		error.getLocation()->getColumnNumber(),
																		XMLStr::convert(error.getMessage())));
			return false;
		}
	}
}

//******************************************************************************
void ErrorReporter::resetErrors()
//******************************************************************************
{
	errors_.clear();
	warnings_.clear();
}
