/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#include "xmlreader.hh"

#include <xercesc/sax/InputSource.hpp>
#include <xercesc/framework/URLInputSource.hpp>
#include <xercesc/framework/LocalFileInputSource.hpp>
#include <xercesc/framework/MemBufInputSource.hpp>

#include "common.hh"
//#include <iostream>

#ifdef HAVE_NAMESPACES
using namespace std;
XERCESNS_USE
#endif 


//******************************************************************************
XMLConvenientReader::XMLConvenientReader(bool donamespaces, bool doschema, 
																				 bool dovalidation)
		: parser_(0), source_(0)
//******************************************************************************
{
  parser_ = new XercesDOMParser();
	parser_->setDoNamespaces(donamespaces);
	parser_->setDoSchema(doschema);
	parser_->setDoValidation(dovalidation);
	parser_->setCreateEntityReferenceNodes(true);
}

//******************************************************************************
XMLConvenientReader::~XMLConvenientReader()
//******************************************************************************
{
	clearSource();
  delete parser_;
}

//******************************************************************************
void XMLConvenientReader::parse()
//******************************************************************************
{
	ErrorReporter* reporter = new ErrorReporter();
	parser_->setErrorHandler(reporter);

	parser_->parse(*source_);	

	if (reporter->getNErrors()>0)
	{
		throw XMLSyntaxError(reporter);
	}
	else if (reporter->getNWarnings()>0)
	{
		throw XMLSyntaxWarning(reporter);
	}
	else
		delete reporter;
}

//******************************************************************************
bool XMLConvenientReader::setSourceURL(const string& url)
//******************************************************************************
{
	clearSource();
	source_ = new URLInputSource(XMLURL(url.c_str()));

	return (source_);
}

//******************************************************************************
bool XMLConvenientReader::setSourceFile(const string& filename)
//******************************************************************************
{
	XMLStr filepath = filename;

	clearSource();
	source_ = new LocalFileInputSource(filepath);

	return (source_);
}

//******************************************************************************
bool XMLConvenientReader::setSource(istream& stream, int size)
//******************************************************************************
{
	unsigned char* buffer = 0;

 	if (size<0)
 	{
 		int curr = stream.tellg();
 		stream.seekg(0, ios::end);
 		size = stream.tellg();
 		stream.seekg(curr, ios::beg);
 	}

	buffer = new unsigned char[size];
  stream.read((char*) buffer, size);

	return setSource(buffer, size);;
}

//******************************************************************************
bool XMLConvenientReader::setSource(const unsigned char* xmltext, int size)
//******************************************************************************
{
	XMLCh fakeid[] = { 
		chLatin_m, chLatin_e, chLatin_m, chLatin_o, chLatin_r, chLatin_y, chNull };
	
	clearSource();

	source_ = new MemBufInputSource(xmltext, size, fakeid, true);
	return (source_);
}

//******************************************************************************
void XMLConvenientReader::clearSource()
//******************************************************************************
{
	if (source_) delete source_;
	source_ = 0;
}
