/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#include "reader_test.hh"

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <xercesc/dom/DOM.hpp>

#include <xmlutils.hh>
#include <conversion.hh>
#include <common.hh>

#ifdef HAVE_STD
#  include <iostream>
#  include <fstream>
#else
#  include <iostream.h>
#  include <fstream.h>
#endif

#include "templates.hh"

#ifdef HAVE_NAMESPACES
using namespace boost::unit_test_framework;
using namespace std;
XERCESNS_USE
#endif 

//******************************************************************************
void reader_test::test_reading(XMLConvenientReader& reader)
//******************************************************************************
{
	try
	{
		reader.parse();

    //make some priliminary checks
    DOMDocument* doc = reader.getDocument();
		DOMElement* root = doc->getDocumentElement();
    BOOST_MESSAGE(" ** check if the document is really a document");
    BOOST_CHECK(XMLStr::convert(doc->getNodeName())=="#document");
    BOOST_MESSAGE(" ** check if the head node is really a 'personnel'");
    BOOST_CHECK(XMLStr::convert(root->getNodeName())=="personnel");

		doc->release();
	}
	catch(XMLException& xmlerr)
	{
		BOOST_ERROR(" EE received xml error:\n"
								<< XMLStr::convert(xmlerr.getMessage()));
	}
	catch(DOMException& domerr)
	{
		BOOST_ERROR(" EE received DOM error:\n"
								<< domerr.code);
	}
	catch(SyntaxException& synerr)
	{
		BOOST_ERROR(" EE received Syntax error:\n"
								<< synerr.what());
    cerr << XMLUtils::printSyntaxError(synerr.getReport());
	}
}

//******************************************************************************
void reader_test::test_reading_file()
//******************************************************************************
{
	BOOST_MESSAGE("\n" << DELIMITER << "\n" 
								<< " XMLConvenientReader file test " << "\n");
	XMLConvenientReader reader;

	BOOST_MESSAGE(" ** check if personal.xml can be assigned for reading");
	BOOST_CHECK(reader.setSourceFile("personal.xml"));

  test_reading(reader);
}

//******************************************************************************
void reader_test::test_reading_url()
//******************************************************************************
{
  const string testdir = TESTDIR;
 	BOOST_MESSAGE("\n" << DELIMITER << "\n" 
 								<< " XMLConvenientReader file test " << "\n");
	XMLConvenientReader reader;

	BOOST_MESSAGE(" ** check if personal.xml can be assigned for reading");
	BOOST_CHECK(
    reader.setSourceURL(string("file://")+
                        string(TESTDIR)+string("/personal.xml")));

  test_reading(reader);
}

//******************************************************************************
void reader_test::test_reading_stream()
//******************************************************************************
{
	BOOST_MESSAGE("\n" << DELIMITER << "\n" 
								<< " XMLConvenientReader file test " << "\n");
	XMLConvenientReader reader;

  ifstream stream("personal.xml");

	BOOST_MESSAGE(" ** check if personal.xml can be assigned for reading");
	BOOST_CHECK(reader.setSource(stream));

  stream.close();

  test_reading(reader);
}


	void test_reading_stream();

//******************************************************************************
reader_test_suite::reader_test_suite()
//******************************************************************************
: test_suite("reader_test_suite") 
{
	// add member function test cases to a test suite
	boost::shared_ptr<reader_test> instance(new reader_test);
			 
	test_case* filereading_test_case = 
		BOOST_CLASS_TEST_CASE(&reader_test::test_reading_file, instance); 
	test_case* urlreading_test_case = 
		BOOST_CLASS_TEST_CASE(&reader_test::test_reading_url, instance); 
	test_case* streamreading_test_case = 
		BOOST_CLASS_TEST_CASE(&reader_test::test_reading_stream, instance); 
	
	//test case, number of tests that should fail
	add(urlreading_test_case, 0);
	add(filereading_test_case, 0);
	add(streamreading_test_case, 0);
}
