/*******************************************************************************
 Copyright (C) 2001-2004 by Novartis Institutes of Biomedical Research
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the Novartis Open Source License as published by
 Novartis AG; either version 1, or (at your option) any later version of
 the license.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 Novartis Open Source License for more details.
 
*******************************************************************************/

#include "utils.hh"

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <xercesc/dom/DOM.hpp>

#include <xmlutils.hh>
#include <conversion.hh>
#include <common.hh>

#include "templates.hh"

#ifdef HAVE_NAMESPACES
using namespace std;
XERCESNS_USE
#endif 

static int level = 0;

//******************************************************************************
void test_nodes(const DOMNode* const a, const DOMNode* const b, bool verbose)
//******************************************************************************
{
  BOOST_CHECK(a!=b);

  BOOST_MESSAGE(" ** check if the node is named \"" 
                << XMLStr(a->getNodeName()) << "\"");

  if (verbose)
  {
    cout << " DD ";
    for(int i=0;i<level;++i) cout << "  ";
    cout << XMLStr(a->getNodeName()) << " == " 
         << XMLStr(b->getNodeName()) << endl;
  }

  BOOST_CHECK(XMLStr(a->getNodeName())==XMLStr(b->getNodeName()));

  BOOST_MESSAGE(" ** check if the node has the right type");
  BOOST_CHECK(a->getNodeType()==b->getNodeType());
  
  if ((a->getNodeType()==DOMNode::TEXT_NODE) &&
      (b->getNodeType()==DOMNode::TEXT_NODE))
  {

    const DOMText* textnode_a = dynamic_cast<const DOMText*>(a);
    const DOMText* textnode_b = dynamic_cast<const DOMText*>(b);

    XMLStr text_a = textnode_a->getNodeValue();
    XMLStr text_b = textnode_b->getNodeValue();

    if (verbose)
    {
      cout << " DD ";
      for(int i=0;i<level;++i) cout << "  ";
      cout << text_a << " == " << text_b << endl;
    }

    text_a.collapse();
    text_b.collapse();
    
    BOOST_MESSAGE(" ** check if the text-node is \"" << text_a << "\"");
    BOOST_CHECK(text_a==text_b);
  }
 
  if (a->hasChildNodes())
    BOOST_MESSAGE(" ** check if b has children"); 
  else
    BOOST_MESSAGE(" ** check if b has no children"); 
    
  BOOST_CHECK(a->hasChildNodes() == b->hasChildNodes());
  
  if (a->hasChildNodes() && b->hasChildNodes())
  {
    const DOMNode* child_a = a->getFirstChild();
    const DOMNode* child_b = b->getFirstChild();
    
    while (child_a)
    {
      ++level;
      test_nodes(child_a, child_b, verbose);
      --level;
      child_a = child_a->getNextSibling();
      child_b = child_b->getNextSibling();

      if (((child_a) && (!child_b)) || ((!child_a) && (child_b)))
        BOOST_ERROR(" EE Nodes do not have the same number of children ");
    }
  }
}
